@@ -18,15 +18,17 @@
  * (such as via usesIframe()) it's important to re-run a lot of the same tests.
  *
  * @author nicksantos@google.com (Nick Santos)
- * @author jparent@google.com (Julie Parent)
  * @author gboyer@google.com (Garrett Boyer)
  */
 
 /** @suppress {extraProvide} */
 goog.provide('goog.editor.field_test');
 
+goog.require('goog.array');
 goog.require('goog.dom');
 goog.require('goog.dom.Range');
+goog.require('goog.dom.TagName');
+goog.require('goog.dom.classlist');
 goog.require('goog.editor.BrowserFeature');
 goog.require('goog.editor.Field');
 goog.require('goog.editor.Plugin');
@@ -862,15 +864,22 @@ function testNotHandledSelectionChange() {
 function testSelectionChange() {
   var editableField = new FieldConstructor('testField', document);
   var clock = new goog.testing.MockClock(true);
+  var beforeSelectionChanges = goog.testing.recordFunction();
+  goog.events.listen(editableField,
+      goog.editor.Field.EventType.BEFORESELECTIONCHANGE,
+      beforeSelectionChanges);
   var selectionChanges = goog.testing.recordFunction();
   goog.events.listen(editableField, goog.editor.Field.EventType.SELECTIONCHANGE,
       selectionChanges);
 
   editableField.makeEditable();
 
-  // Emulate pressing left arrow key, this should result in SELECTIONCHANGE
-  // event after a short timeout.
+  // Emulate pressing left arrow key, this should result in a
+  // BEFORESELECTIONCHANGE event immediately, and a SELECTIONCHANGE event after
+  // a short timeout.
   editableField.handleKeyUp_({keyCode: goog.events.KeyCodes.LEFT});
+  assertEquals('Before selection change should fire immediately', 1,
+      beforeSelectionChanges.getCallCount());
   assertEquals('Selection change should be on a timer', 0,
       selectionChanges.getCallCount());
   clock.tick(1000);
@@ -892,6 +901,10 @@ function testSelectionChangeOnMouseUp() {
       new goog.events.BrowserEvent({type: 'mouseup', target: 'fakeTarget'});
   var editableField = new FieldConstructor('testField', document);
   var clock = new goog.testing.MockClock(true);
+  var beforeSelectionChanges = goog.testing.recordFunction();
+  goog.events.listen(editableField,
+      goog.editor.Field.EventType.BEFORESELECTIONCHANGE,
+      beforeSelectionChanges);
   var selectionChanges = goog.testing.recordFunction();
   goog.events.listen(editableField, goog.editor.Field.EventType.SELECTIONCHANGE,
       selectionChanges);
@@ -902,9 +915,12 @@ function testSelectionChangeOnMouseUp() {
 
   editableField.makeEditable();
 
-  // Emulate a mouseup event, this should result in an immediate
-  // SELECTIONCHANGE, plus a second one in IE after a short timeout.
+  // Emulate a mouseup event, this should result in immediate
+  // BEFORESELECTIONCHANGE and SELECTIONCHANGE, plus a second SELECTIONCHANGE in
+  // IE after a short timeout.
   editableField.handleMouseUp_(fakeEvent);
+  assertEquals('Before selection change should fire immediately', 1,
+      beforeSelectionChanges.getCallCount());
   assertEquals('Selection change should fire immediately', 1,
       selectionChanges.getCallCount());
   assertEquals('Plugin should have handled selection change immediately', 1,
@@ -1109,7 +1125,7 @@ function testRestoreSavedRange() {
 
   // Create another node to take the focus later.
   var doc = goog.dom.getOwnerDocument(editableField.getElement());
-  var otherElem = doc.createElement('div');
+  var otherElem = doc.createElement(goog.dom.TagName.DIV);
   otherElem.tabIndex = '1';  // Make it focusable.
   editableField.getElement().parentNode.appendChild(otherElem);
 
@@ -1306,7 +1322,7 @@ function testHandleWindowLevelMouseUp() {
         selectionHasFired = true;
       });
   var editableElement = editableField.getElement();
-  var otherElement = goog.dom.createDom('div');
+  var otherElement = goog.dom.createDom(goog.dom.TagName.DIV);
   goog.dom.insertSiblingAfter(otherElement, document.body.lastChild);
 
   goog.testing.events.fireMouseDownEvent(editableElement);
@@ -1326,7 +1342,7 @@ function testNoHandleWindowLevelMouseUp() {
         selectionHasFired = true;
       });
   var editableElement = editableField.getElement();
-  var otherElement = goog.dom.createDom('div');
+  var otherElement = goog.dom.createDom(goog.dom.TagName.DIV);
   goog.dom.insertSiblingAfter(otherElement, document.body.lastChild);
 
   goog.testing.events.fireMouseDownEvent(editableElement);
@@ -1355,3 +1371,22 @@ function testIsGeneratingKey() {
     assertFalse(goog.editor.Field.isGeneratingKey_(imeKeyEvent, false));
   }
 }
+
+function testSetEditableClassName() {
+  var element = goog.dom.getElement('testField');
+  var editableField = new FieldConstructor('testField');
+
+  assertFalse(goog.dom.classlist.contains(element, 'editable'));
+  editableField.makeEditable();
+  assertTrue(goog.dom.classlist.contains(element, 'editable'));
+  assertEquals(1, goog.array.count(
+      goog.dom.classlist.get(element), goog.functions.equalTo('editable')));
+
+  // Skip restore won't reset the original element's CSS classes.
+  editableField.makeUneditable(true /* opt_skipRestore */);
+
+  editableField.makeEditable();
+  assertTrue(goog.dom.classlist.contains(element, 'editable'));
+  assertEquals(1, goog.array.count(
+      goog.dom.classlist.get(element), goog.functions.equalTo('editable')));
+}
