@@ -12,8 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-goog.provide('goog.ui.editor.LinkDialogTest');
-goog.setTestOnly('goog.ui.editor.LinkDialogTest');
+goog.provide('goog.ui.editor.plugins.LinkDialogTest');
+goog.setTestOnly('goog.ui.editor.plugins.LinkDialogTest');
 
 goog.require('goog.dom');
 goog.require('goog.dom.DomHelper');
@@ -682,7 +682,7 @@ function testBug7279077ScrollOnFocus() {
   elem.style.contenteditable = 'true';
 
   // Add a bunch of text before the anchor tag.
-  var longTextElem = document.createElement('span');
+  var longTextElem = document.createElement(goog.dom.TagName.SPAN);
   longTextElem.innerHTML = goog.string.repeat('All work and no play.<p>', 20);
   elem.insertBefore(longTextElem, elem.firstChild);
 
@@ -723,7 +723,7 @@ function testBug7279077ScrollOnFocus() {
  * it.
  */
 function setUpRealEditableField() {
-  fieldElem = document.createElement('div');
+  fieldElem = document.createElement(goog.dom.TagName.DIV);
   fieldElem.id = 'myField';
   document.body.appendChild(fieldElem);
   fieldElem.appendChild(anchorElem);
