@@ -16,6 +16,7 @@
  * @fileoverview TrogEdit plugin to handle enter keys by inserting the
  * specified block level tag.
  *
+ * @author robbyw@google.com (Robby Walker)
  */
 
 goog.provide('goog.editor.plugins.TagOnEnterHandler');
@@ -30,7 +31,8 @@ goog.require('goog.editor.plugins.EnterHandler');
 goog.require('goog.editor.range');
 goog.require('goog.editor.style');
 goog.require('goog.events.KeyCodes');
-goog.require('goog.string');
+goog.require('goog.functions');
+goog.require('goog.string.Unicode');
 goog.require('goog.style');
 goog.require('goog.userAgent');
 
@@ -523,10 +525,10 @@ goog.editor.plugins.TagOnEnterHandler.prototype.handleRegularEnterGecko_ =
       // it out of the anchor.
       var anchorToRemove = goog.editor.node.isEmpty(leftAnchor, false) ?
           leftAnchor : rightAnchor;
-      goog.dom.flattenElement(/** @type {Element} */ (anchorToRemove));
+      goog.dom.flattenElement(/** @type {!Element} */ (anchorToRemove));
     }
   }
-  return /** @type {Element} */ (newNode);
+  return /** @type {!Element} */ (newNode);
 };
 
 
@@ -551,7 +553,8 @@ goog.editor.plugins.TagOnEnterHandler.prototype.scrollCursorIntoViewGecko_ =
 
   // Determine the height of that element, since we want the bottom of the
   // element to be in view.
-  var bottomOfNode = elementY + element.offsetHeight;
+  var bottomOfNode = elementY +
+      /** @type {!HTMLElement} */ (element).offsetHeight;
 
   var dom = this.getFieldDomHelper();
   var win = this.getFieldDomHelper().getWindow();
