@@ -16,13 +16,13 @@
 /**
  * @fileoverview CanvasGraphics sub class that uses the canvas tag for drawing.
  * @author robbyw@google.com (Robby Walker)
- * @author wcrosby@google.com (Wayne Crosby)
  */
 
 
 goog.provide('goog.graphics.CanvasGraphics');
 
 
+goog.require('goog.dom.TagName');
 goog.require('goog.events.EventType');
 goog.require('goog.graphics.AbstractGraphics');
 goog.require('goog.graphics.CanvasEllipseElement');
@@ -92,7 +92,10 @@ goog.graphics.CanvasGraphics.prototype.setElementStroke = function(
 
 
 /**
- * Set the transformation of an element.
+ * Set the translation and rotation of an element.
+ *
+ * If a more general affine transform is needed than this provides
+ * (e.g. skew and scale) then use setElementAffineTransform.
  * @param {goog.graphics.Element} element The element wrapper.
  * @param {number} x The x coordinate of the translation transform.
  * @param {number} y The y coordinate of the translation transform.
@@ -107,6 +110,22 @@ goog.graphics.CanvasGraphics.prototype.setElementTransform = function(element,
 };
 
 
+/**
+ * Set the transformation of an element.
+ *
+ * Note that in this implementation this method just calls this.redraw()
+ * and the affineTransform param is unused.
+ * @param {!goog.graphics.Element} element The element wrapper.
+ * @param {!goog.graphics.AffineTransform} affineTransform The
+ *     transformation applied to this element.
+ * @override
+ */
+goog.graphics.CanvasGraphics.prototype.setElementAffineTransform =
+    function(element, affineTransform) {
+  this.redraw();
+};
+
+
 /**
  * Push an element transform on to the transform stack.
  * @param {goog.graphics.Element} element The transformed element.
@@ -145,11 +164,11 @@ goog.graphics.CanvasGraphics.prototype.popElementTransform = function() {
  * @override
  */
 goog.graphics.CanvasGraphics.prototype.createDom = function() {
-  var element = this.dom_.createDom('div',
+  var element = this.dom_.createDom(goog.dom.TagName.DIV,
       {'style': 'position:relative;overflow:hidden'});
   this.setElementInternal(element);
 
-  this.canvas_ = this.dom_.createDom('canvas');
+  this.canvas_ = this.dom_.createDom(goog.dom.TagName.CANVAS);
   element.appendChild(this.canvas_);
 
   /**
@@ -402,20 +421,6 @@ goog.graphics.CanvasGraphics.prototype.drawElement = function(element) {
 };
 
 
-/**
- * Append an element.
- *
- * @param {goog.graphics.Element} element The element to draw.
- * @param {goog.graphics.CanvasGroupElement|undefined} group The group to draw
- *     it in. If null or undefined, defaults to the root group.
- * @private
- * @deprecated Use append instead.
- */
-goog.graphics.CanvasGraphics.prototype.append_ = function(element, group) {
-  this.append(element, group);
-};
-
-
 /**
  * Append an element.
  *
