@@ -16,7 +16,6 @@
 /**
  * @fileoverview VmlGraphics sub class that uses VML to draw the graphics.
  * @author arv@google.com (Erik Arvidsson)
- * @author yoah@google.com (Yoah Bar-David)
  */
 
 
@@ -24,6 +23,8 @@ goog.provide('goog.graphics.VmlGraphics');
 
 
 goog.require('goog.array');
+goog.require('goog.dom.TagName');
+goog.require('goog.dom.safe');
 goog.require('goog.events');
 goog.require('goog.events.EventHandler');
 goog.require('goog.events.EventType');
@@ -37,9 +38,11 @@ goog.require('goog.graphics.VmlImageElement');
 goog.require('goog.graphics.VmlPathElement');
 goog.require('goog.graphics.VmlRectElement');
 goog.require('goog.graphics.VmlTextElement');
+goog.require('goog.html.uncheckedconversions');
 goog.require('goog.math');
 goog.require('goog.math.Size');
 goog.require('goog.string');
+goog.require('goog.string.Const');
 goog.require('goog.style');
 
 
@@ -70,6 +73,7 @@ goog.graphics.VmlGraphics = function(width, height,
                                       opt_coordWidth, opt_coordHeight,
                                       opt_domHelper);
   this.handler_ = new goog.events.EventHandler(this);
+  this.registerDisposable(this.handler_);
 };
 goog.inherits(goog.graphics.VmlGraphics, goog.graphics.AbstractGraphics);
 
@@ -103,8 +107,9 @@ goog.graphics.VmlGraphics.VML_IMPORT_ = '#default#VML';
  * @private
  * @type {boolean}
  */
-goog.graphics.VmlGraphics.IE8_MODE_ = document.documentMode &&
-    document.documentMode >= 8;
+goog.graphics.VmlGraphics.IE8_MODE_ =
+    goog.global.document && goog.global.document.documentMode &&
+    goog.global.document.documentMode >= 8;
 
 
 /**
@@ -247,7 +252,16 @@ goog.graphics.VmlGraphics.prototype.getVmlElement = function(id) {
  */
 goog.graphics.VmlGraphics.prototype.updateGraphics_ = function() {
   if (goog.graphics.VmlGraphics.IE8_MODE_ && this.isInDocument()) {
-    this.getElement().innerHTML = this.getElement().innerHTML;
+    // There's a risk of mXSS here, as the browser is not guaranteed to
+    // return the HTML that was originally written, when innerHTML is read.
+    // However, given that this a deprecated API and affects only IE, it seems
+    // an acceptable risk.
+    var html = goog.html.uncheckedconversions
+        .safeHtmlFromStringKnownToSatisfyTypeContract(
+            goog.string.Const.from('Assign innerHTML to itself'),
+            this.getElement().innerHTML);
+    goog.dom.safe.setInnerHtml(
+        /** @type {!Element} */ (this.getElement()), html);
   }
 };
 
@@ -275,7 +289,7 @@ goog.graphics.VmlGraphics.prototype.append_ = function(element, opt_group) {
  */
 goog.graphics.VmlGraphics.prototype.setElementFill = function(element, fill) {
   var vmlElement = element.getElement();
-  this.removeFill(vmlElement);
+  goog.graphics.VmlGraphics.removeFill_(vmlElement);
   if (fill instanceof goog.graphics.SolidFill) {
     // NOTE(arv): VML does not understand 'transparent' so hard code support
     // for it.
@@ -339,19 +353,13 @@ goog.graphics.VmlGraphics.prototype.setElementStroke = function(element,
     }
 
     var strokeElement = vmlElement.getElementsByTagName('stroke')[0];
-    if (width < 1) {
+    if (!strokeElement) {
       strokeElement = strokeElement || this.createVmlElement('stroke');
-      strokeElement.opacity = width;
-      strokeElement.weight = '1px';
-      strokeElement.color = stroke.getColor();
       vmlElement.appendChild(strokeElement);
-    } else {
-      if (strokeElement) {
-        vmlElement.removeChild(strokeElement);
-      }
-      vmlElement.strokecolor = stroke.getColor();
-      vmlElement.strokeweight = width + 'px';
     }
+    strokeElement.opacity = stroke.getOpacity();
+    strokeElement.weight = width + 'px';
+    strokeElement.color = stroke.getColor();
   } else {
     vmlElement.stroked = false;
   }
@@ -360,7 +368,10 @@ goog.graphics.VmlGraphics.prototype.setElementStroke = function(element,
 
 
 /**
- * Set the transformation of an element.
+ * Set the translation and rotation of an element.
+ *
+ * If a more general affine transform is needed than this provides
+ * (e.g. skew and scale) then use setElementAffineTransform.
  * @param {goog.graphics.Element} element The element wrapper.
  * @param {number} x The x coordinate of the translation transform.
  * @param {number} y The y coordinate of the translation transform.
@@ -383,19 +394,62 @@ goog.graphics.VmlGraphics.prototype.setElementTransform = function(element, x,
 };
 
 
+/**
+ * Set the transformation of an element.
+ * @param {!goog.graphics.Element} element The element wrapper.
+ * @param {!goog.graphics.AffineTransform} affineTransform The
+ *     transformation applied to this element.
+ * @override
+ */
+goog.graphics.VmlGraphics.prototype.setElementAffineTransform = function(
+    element, affineTransform) {
+  var t = affineTransform;
+  var vmlElement = element.getElement();
+  goog.graphics.VmlGraphics.removeSkew_(vmlElement);
+  var skewNode = this.createVmlElement('skew');
+  skewNode.on = 'true';
+  // Move the transform origin to 0px,0px of the graphics.
+  // In VML, 0,0 means the center of the element, -0.5,-0.5 left top conner of
+  // it.
+  skewNode.origin =
+      (-vmlElement.style.pixelLeft / vmlElement.style.pixelWidth - 0.5) + ',' +
+      (-vmlElement.style.pixelTop / vmlElement.style.pixelHeight - 0.5);
+  skewNode.offset = t.getTranslateX().toFixed(1) + 'px,' +
+                    t.getTranslateY().toFixed(1) + 'px';
+  skewNode.matrix = [t.getScaleX().toFixed(6), t.getShearX().toFixed(6),
+                     t.getShearY().toFixed(6), t.getScaleY().toFixed(6),
+                     0, 0].join(',');
+  vmlElement.appendChild(skewNode);
+  this.updateGraphics_();
+};
+
+
+/**
+ * Removes the skew information from a dom element.
+ * @param {Element} element DOM element.
+ * @private
+ */
+goog.graphics.VmlGraphics.removeSkew_ = function(element) {
+  goog.array.forEach(element.childNodes, function(child) {
+    if (child.tagName == 'skew') {
+      element.removeChild(child);
+    }
+  });
+};
+
+
 /**
  * Removes the fill information from a dom element.
  * @param {Element} element DOM element.
+ * @private
  */
-goog.graphics.VmlGraphics.prototype.removeFill = function(element) {
+goog.graphics.VmlGraphics.removeFill_ = function(element) {
   element.fillcolor = '';
-  var v = element.childNodes.length;
-  for (var i = 0; i < element.childNodes.length; i++) {
-    var child = element.childNodes[i];
+  goog.array.forEach(element.childNodes, function(child) {
     if (child.tagName == 'fill') {
       element.removeChild(child);
     }
-  }
+  });
 };
 
 
@@ -485,7 +539,7 @@ goog.graphics.VmlGraphics.prototype.createDom = function() {
   // All inner elements are absolutly positioned on-top of this div.
   var pixelWidth = this.width;
   var pixelHeight = this.height;
-  var divElement = this.dom_.createDom('div', {
+  var divElement = this.dom_.createDom(goog.dom.TagName.DIV, {
     'style': 'overflow:hidden;position:relative;width:' +
         goog.graphics.VmlGraphics.toCssSize(pixelWidth) + ';height:' +
         goog.graphics.VmlGraphics.toCssSize(pixelHeight)
