@@ -182,6 +182,7 @@ fi
 
     install -d -m 0755 "${DESTDIR}${rundir}"
     install -d -m 700 "${DESTDIR}${app_logdir}"
+    install -d -m 700 "${DESTDIR}/opt/puppetlabs/server/data/puppetserver/jars"
 }
 
 function task_install_redhat {
@@ -319,6 +320,8 @@ function task_postinst_permissions {
     chmod 750 $projconfdir
     chown puppet:puppet $rundir
     chmod 0755 $rundir
+    chown puppet:puppet /opt/puppetlabs/server/data/puppetserver/jars
+    chmod 700 /opt/puppetlabs/server/data/puppetserver/jars
 }
 
 # Install logrotate (usually el7, fedora 16 and above)
