@@ -8,8 +8,7 @@
   (:import [org.ini4j Ini Config BasicProfileSection]
            [javax.naming.ldap LdapName]
            [java.io StringWriter Reader File])
-  (:require [clojure.test]
-            [clojure.tools.logging :as log]
+  (:require [clojure.tools.logging :as log]
             [clojure.string :as string]
             [clojure.tools.cli :as cli]
             [clojure.java.io :as io]
@@ -21,10 +20,16 @@
         [clojure.string :only (split)]
         [clojure.stacktrace :only (print-cause-trace)]
         [clojure.pprint :only [pprint]]
-        [clj-time.core :only [now]]
+        [clj-time.core :only [now seconds minutes hours days years]]
         [clj-time.coerce :only [ICoerce to-date-time]]
         [clj-time.format :only [formatters unparse]]))
 
+
+(defn error-map
+  [kind message]
+  {:kind kind
+   :msg message})
+
 ;; ## Type checking
 
 (defn array?
@@ -70,8 +75,8 @@ to be a zipper."
   (condp = (.toLowerCase s)
     "true" true
     "false" false
-    (throw+ {:type ::parse-error
-             :message (format "Unable to parse '%s' to a boolean" s)})))
+    (throw+ (error-map ::parse-error
+                       (format "Unable to parse '%s' to a boolean" s)))))
 
 (defn parse-bool
   "Parse a string and return its boolean value."
@@ -145,10 +150,9 @@ to be a zipper."
 (defn lines
   "Returns a sequence of lines from the given filename"
   [filename]
-  (-> filename
-    (fs/file)
-    (reader)
-    (line-seq)))
+  (with-open [file-reader (reader (fs/file filename))]
+    ;; line seq is lazy and file-reader gets closed
+    (doall (line-seq file-reader))))
 
 (defn mkdirs!
   "Given a path (may be a File or a string), creates a directory (including any
@@ -162,33 +166,33 @@ to be a zipper."
 
   The slingshot exception will look like this:
 
-  `{:type     :puppetlabs.kitchensink.core/io-error
-    :message  \"Parent directory '/foo/bar' is not writable\"}`"
+  `{:kind     :puppetlabs.kitchensink.core/io-error
+    :msg      \"Parent directory '/foo/bar' is not writable\"}`"
   [path]
   {:pre [((some-fn #(instance? File %) string?) path)]
    :post [(fs/directory? path)]}
   (let [path-as-file (fs/file path)]
     (if (fs/file? path-as-file)
-      (throw+ {:type    ::io-error
-               :message (format "Path '%s' is a file" path)})
+      (throw+ (error-map ::io-error
+                         (format "Path '%s' is a file" path)))
       (doseq [^File dir (reverse (cons path-as-file (fs/parents path-as-file)))]
         (when-not (fs/exists? dir)
           (let [parent (.getParentFile dir)]
             (when (fs/file? parent)
-              (throw+ {:type ::io-error
-                       :message (format "Parent directory '%s' is a file"
-                                        parent)}))
+              (throw+ (error-map ::io-error
+                                 (format "Parent directory '%s' is a file"
+                                         parent))))
 
             (when-not (.canWrite parent)
-              (throw+ {:type ::io-error
-                       :message (format "Parent directory '%s' is not writable"
-                                        parent)}))
+              (throw+ (error-map ::io-error
+                                 (format "Parent directory '%s' is not writable"
+                                         parent))))
 
             (let [success (.mkdir dir)]
               (when-not success
-                (throw+ {:type ::io-error
-                         :message (format "Unable to create directory '%s'"
-                                          parent)})))))))))
+                (throw+ (error-map ::io-error
+                                   (format "Unable to create directory '%s'"
+                                           parent)))))))))))
 
 ;; ## Math
 
@@ -254,6 +258,48 @@ to be a zipper."
       (last weights-and-values)
       (-> (first selected) second))))
 
+(def ascii-character-sets
+  (let [concatv (comp vec concat)
+        ALPHA (mapv char (range 65 91))
+        alpha (mapv char (range 97 123))
+        digits (mapv char (range 48 58))
+        symbols (concatv (map char (range 33 48))
+                         (map char (range 91 97))
+                         (map char (range 123 127)))]
+    {:alpha (concatv alpha ALPHA)
+     :alpha-lower alpha
+     :alpha-upper ALPHA
+     :alpha-digits (concatv alpha ALPHA digits)
+     :alpha-digits-symbols (concatv alpha ALPHA digits symbols)
+     :symbols symbols
+     :digits digits}))
+
+(defn rand-str
+  "Produces a random string of length n, drawn from the given collection of
+  characters. The following keywords may be used in place of a character
+  collection:
+    :alpha - [a-zA-Z]
+    :alpha-lower - [a-z]
+    :alpha-upper - [A-Z]
+    :alpha-digits - [a-zA-Z0-9]
+    :alpha-digits-symbols - all printable ASCII characters besides space
+    :symbols - all visible, non-alpha-numeric ASCII characters (no space)
+    :digits - [0-9]
+  If no character collection or keyword is provided, :alpha-digits-symbols is
+  used by default."
+  ([n] (rand-str :alpha-digits-symbols n))
+  ([characters n]
+   (let [char-coll (cond
+                     (and (keyword? characters) (contains? ascii-character-sets characters))
+                     (get ascii-character-sets characters)
+
+                     (keyword? characters)
+                     (throw (IllegalArgumentException.
+                              (str characters " is not a recognized character collection keyword")))
+
+                     :else (vec characters))]
+     (apply str (repeatedly n #(rand-nth char-coll))))))
+
 ;; ## Collection operations
 
 (defn symmetric-difference
@@ -718,12 +764,13 @@ to be a zipper."
           (every? keyword? (keys %))
           (every? map? (vals %))]}
 
-  (reduce (fn [acc [name section]]
-            (assoc acc
-              (keywordize name)
-              (create-section-map section)))
-          {}
-          (parse-ini (reader filename))))
+  (with-open [ini (reader filename)]
+    (reduce (fn [acc [name section]]
+              (assoc acc
+                     (keywordize name)
+                     (create-section-map section)))
+            {}
+            (parse-ini ini))))
 
 (defn inis-to-map
   "Takes a path and converts the pointed-at .ini files into a nested
@@ -805,7 +852,7 @@ to be a zipper."
                the program to exit and display the help message.
 
   ** The map is thrown using 'slingshot' (https://github.com/scgilardi/slingshot).
-  It contains a `:type` and `:message`, where type is either `:error` or `:help`,
+  It contains a `:kind` and `:msg`, where type is either `:error` or `:help`,
   and the message is either the error message or a help banner.
 
   Returns a three-item vector, containing:
@@ -825,19 +872,16 @@ to be a zipper."
                   (apply str errors)
                   "\n\n"
                   summary)]
-        (throw+ {:type ::cli-error
-                 :message msg})))
+        (throw+ (error-map ::cli-error msg))))
     (when (:help options)
-      (throw+ {:type ::cli-help
-               :message summary}))
+      (throw+ (error-map ::cli-help summary)))
     (when-let [missing-field (some #(if (not (contains? options %)) %) required-args)]
       (let [msg (str
                   "\n\n"
                   (format "Missing required argument '--%s'!" (name missing-field))
                   "\n\n"
                   summary)]
-        (throw+ {:type ::cli-error
-                 :message msg})))
+        (throw+ (error-map ::cli-error msg))))
     [options arguments summary])))
 
 
@@ -921,6 +965,24 @@ to be a zipper."
   (let [bytes (.getBytes s "UTF-8")]
     (digest/sha-1 [bytes])))
 
+(defn utf8-string->sha256
+  "Compute a SHA-256 hash for the UTF-8 encoded version of the supplied
+  string"
+  [^String s]
+  {:pre  [(string? s)]
+   :post [(string? %)]}
+  (let [bytes (.getBytes s "UTF-8")]
+    (digest/sha-256 [bytes])))
+
+(defn file->sha256
+  "Compute a SHA-256 hash for the given java.io.File object.
+  Uses an InputStream to read the file, so it doesn't load all the contents into
+  memory at once."
+  [^java.io.File file]
+  {:pre  [(instance? java.io.File file)]
+   :post [(string? %)]}
+  (digest/sha-256 file))
+
 (defn bounded-memoize
   "Similar to memoize, but the cache will be reset if the number of entries
   exceeds the specified `bound`."
@@ -1022,3 +1084,70 @@ to be a zipper."
     `(let [~g ~expr
            ~@(interleave (repeat g) (map pstep forms))]
        ~g)))
+
+(defn- port-open?
+  [port]
+  (try
+    (with-open [_ (java.net.ServerSocket. port)]
+      port)
+    (catch java.net.BindException e
+      (when-not (re-find #"already in use" (.getMessage e))
+        (throw e)))))
+
+(defn open-port-num
+  "Returns a currently open port number in the traditional ephemeral port range
+  of 49152 through 65535."
+  []
+  (let [lo 49152
+        hi 65536] ; one higher because the upper limit is exclusive
+    (if-let [open-port (some port-open? (shuffle (range lo hi)))]
+      open-port
+      (throw (java.net.BindException.
+               "All ephemeral ports are already in use (Bind failed)")))))
+
+(defmacro assoc-if-new
+  "Assocs the provided values with the corresponding keys if and only
+  if the key is not already present in map."
+  [map key val & kvs]
+  {:pre [(even? (count kvs))]}
+  (let [deferred-kvs (vec (for [[k v] (cons [key val] (partition 2 kvs))]
+                            [k `(fn [] ~v)]))]
+    `(let [updates# (for [[k# v#] ~deferred-kvs
+                          :when (= ::not-found (get ~map k# ::not-found))]
+                      [k# (v#)])]
+       (merge ~map (into {} updates#)))))
+
+(defn deref-swap!
+  "Like swap! but returns the old value.
+   Adapted from http://stackoverflow.com/a/15442107."
+  [atom f & args]
+  (loop []
+    (let [old @atom
+          new (apply f old args)]
+      (if (compare-and-set! atom old new)
+        old
+        (recur)))))
+
+(defn parse-interval
+  "Given a time string of the form \"<number>\", or \"<number><unit>\", this
+  function parses this time amount and returns a joda time Period instance.
+  If the unit is left off, the units are assumed to be seconds
+
+  Example: \"12h\" -> (clj-time.core/hours 12)
+  Example: \"12\" -> (clj-time.core/seconds 12)
+
+  Possible units: s(econds), m(inutes), h(ours), d(ays), y(ears)
+
+  Returns nil if the time string cannot be parsed."
+  [time-str]
+  (when-not (nil? time-str)
+    (when-let [[_ num unit] (re-matches #"^(\d+)([smhdy]?)$" time-str)]
+      (let [num (parse-int num)
+            time-fn (case unit
+                      "s" seconds
+                      "m" minutes
+                      "h" hours
+                      "d" days
+                      "y" years
+                      "" seconds)]
+        (time-fn num)))))
