@@ -140,10 +140,14 @@ goog.require('goog.Uri');
 goog.require('goog.asserts');
 goog.require('goog.debug');
 goog.require('goog.dom');
+goog.require('goog.dom.InputType');
+goog.require('goog.dom.TagName');
+goog.require('goog.dom.safe');
 goog.require('goog.events');
 goog.require('goog.events.Event');
 goog.require('goog.events.EventTarget');
 goog.require('goog.events.EventType');
+goog.require('goog.html.uncheckedconversions');
 goog.require('goog.json');
 goog.require('goog.log');
 goog.require('goog.log.Level');
@@ -151,6 +155,7 @@ goog.require('goog.net.ErrorCode');
 goog.require('goog.net.EventType');
 goog.require('goog.reflect');
 goog.require('goog.string');
+goog.require('goog.string.Const');
 goog.require('goog.structs');
 goog.require('goog.userAgent');
 
@@ -174,7 +179,7 @@ goog.net.IframeIo = function() {
   /**
    * An array of iframes that have been finished with.  We need them to be
    * disposed async, so we don't confuse the browser (see below).
-   * @type {Array.<Element>}
+   * @type {Array<Element>}
    * @private
    */
   this.iframesForDisposal_ = [];
@@ -324,8 +329,8 @@ goog.net.IframeIo.getNextName_ = function() {
  */
 goog.net.IframeIo.getForm_ = function() {
   if (!goog.net.IframeIo.form_) {
-    goog.net.IframeIo.form_ =
-        /** @type {HTMLFormElement} */(goog.dom.createDom('form'));
+    goog.net.IframeIo.form_ = /** @type {!HTMLFormElement} */ (
+        goog.dom.createDom(goog.dom.TagName.FORM));
     goog.net.IframeIo.form_.acceptCharset = 'utf-8';
 
     // Hide the form and move it off screen
@@ -351,8 +356,8 @@ goog.net.IframeIo.getForm_ = function() {
 goog.net.IframeIo.addFormInputs_ = function(form, data) {
   var helper = goog.dom.getDomHelper(form);
   goog.structs.forEach(data, function(value, key) {
-    var inp = helper.createDom('input',
-        {'type': 'hidden', 'name': key, 'value': value});
+    var inp = helper.createDom(goog.dom.TagName.INPUT,
+        {'type': goog.dom.InputType.HIDDEN, 'name': key, 'value': value});
     form.appendChild(inp);
   });
 };
@@ -459,24 +464,6 @@ goog.net.IframeIo.prototype.lastContent_ = null;
 goog.net.IframeIo.prototype.lastErrorCode_ = goog.net.ErrorCode.NO_ERROR;
 
 
-/**
- * Number of milliseconds after which an incomplete request will be aborted and
- * a {@link goog.net.EventType.TIMEOUT} event raised; 0 means no timeout is set.
- * @type {number}
- * @private
- */
-goog.net.IframeIo.prototype.timeoutInterval_ = 0;
-
-
-/**
- * Window timeout ID used to cancel the timeout event handler if the request
- * completes successfully.
- * @type {?number}
- * @private
- */
-goog.net.IframeIo.prototype.timeoutId_ = null;
-
-
 /**
  * Window timeout ID used to detect when firefox silently fails.
  * @type {?number}
@@ -511,6 +498,18 @@ goog.net.IframeIo.prototype.errorHandled_;
 goog.net.IframeIo.prototype.ignoreResponse_ = false;
 
 
+/** @private {Function} */
+goog.net.IframeIo.prototype.errorChecker_;
+
+
+/** @private {Object} */
+goog.net.IframeIo.prototype.lastCustomError_;
+
+
+/** @private {?string} */
+goog.net.IframeIo.prototype.lastContentHtml_;
+
+
 /**
  * Sends a request via an iframe.
  *
@@ -793,28 +792,6 @@ goog.net.IframeIo.prototype.getErrorChecker = function() {
 };
 
 
-/**
- * Returns the number of milliseconds after which an incomplete request will be
- * aborted, or 0 if no timeout is set.
- * @return {number} Timeout interval in milliseconds.
- */
-goog.net.IframeIo.prototype.getTimeoutInterval = function() {
-  return this.timeoutInterval_;
-};
-
-
-/**
- * Sets the number of milliseconds after which an incomplete request will be
- * aborted and a {@link goog.net.EventType.TIMEOUT} event raised; 0 means no
- * timeout is set.
- * @param {number} ms Timeout interval in milliseconds; 0 means none.
- */
-goog.net.IframeIo.prototype.setTimeoutInterval = function(ms) {
-  // TODO (pupius) - never used - doesn't look like timeouts were implemented
-  this.timeoutInterval_ = Math.max(0, ms);
-};
-
-
 /**
  * @return {boolean} Whether the server response is being ignored.
  */
@@ -898,19 +875,19 @@ goog.net.IframeIo.prototype.sendFormInternal_ = function() {
 
     // Open and document.write another iframe into the iframe
     var doc = goog.dom.getFrameContentDocument(this.iframe_);
-    var html = '<body><iframe id=' + innerFrameName +
-               ' name=' + innerFrameName + '></iframe>';
+    var html;
     if (document.baseURI) {
       // On Safari 4 and 5 the new iframe doesn't inherit the current baseURI.
-      html = '<head><base href="' + goog.string.htmlEscape(document.baseURI) +
-             '"></head>' + html;
+      html = goog.net.IframeIo.createIframeHtmlWithBaseUri_(innerFrameName);
+    } else {
+      html = goog.net.IframeIo.createIframeHtml_(innerFrameName);
     }
     if (goog.userAgent.OPERA) {
       // Opera adds a history entry when document.write is used.
       // Change the innerHTML of the page instead.
-      doc.documentElement.innerHTML = html;
+      goog.dom.safe.setInnerHtml(doc.documentElement, html);
     } else {
-      doc.write(html);
+      goog.dom.safe.documentWrite(doc, html);
     }
 
     // Listen for the iframe's load
@@ -920,7 +897,7 @@ goog.net.IframeIo.prototype.sendFormInternal_ = function() {
     }
 
     // Fix text areas, since importNode won't clone changes to the value
-    var textareas = this.form_.getElementsByTagName('textarea');
+    var textareas = this.form_.getElementsByTagName(goog.dom.TagName.TEXTAREA);
     for (var i = 0, n = textareas.length; i < n; i++) {
       // The childnodes represent the initial child nodes for the text area
       // appending a text node essentially resets the initial value ready for
@@ -940,11 +917,13 @@ goog.net.IframeIo.prototype.sendFormInternal_ = function() {
     doc.body.appendChild(clone);
 
     // Fix select boxes, importNode won't override the default value
-    var selects = this.form_.getElementsByTagName('select');
-    var clones = clone.getElementsByTagName('select');
+    var selects = this.form_.getElementsByTagName(goog.dom.TagName.SELECT);
+    var clones = clone.getElementsByTagName(goog.dom.TagName.SELECT);
     for (var i = 0, n = selects.length; i < n; i++) {
-      var selectsOptions = selects[i].getElementsByTagName('option');
-      var clonesOptions = clones[i].getElementsByTagName('option');
+      var selectsOptions = selects[i].getElementsByTagName(
+          goog.dom.TagName.OPTION);
+      var clonesOptions = clones[i].getElementsByTagName(
+          goog.dom.TagName.OPTION);
       for (var j = 0, m = selectsOptions.length; j < m; j++) {
         clonesOptions[j].selected = selectsOptions[j].selected;
       }
@@ -954,10 +933,10 @@ goog.net.IframeIo.prototype.sendFormInternal_ = function() {
     // attribute for <input type="file"> nodes, which results in an empty
     // upload if the clone is submitted.  Check, and if the clone failed, submit
     // using the original form instead.
-    var inputs = this.form_.getElementsByTagName('input');
-    var inputClones = clone.getElementsByTagName('input');
+    var inputs = this.form_.getElementsByTagName(goog.dom.TagName.INPUT);
+    var inputClones = clone.getElementsByTagName(goog.dom.TagName.INPUT);
     for (var i = 0, n = inputs.length; i < n; i++) {
-      if (inputs[i].type == 'file') {
+      if (inputs[i].type == goog.dom.InputType.FILE) {
         if (inputs[i].value != inputClones[i].value) {
           goog.log.fine(this.logger_,
               'File input value not cloned properly.  Will ' +
@@ -1006,6 +985,41 @@ goog.net.IframeIo.prototype.sendFormInternal_ = function() {
 };
 
 
+/**
+ * @param {string} innerFrameName
+ * @return {!goog.html.SafeHtml}
+ * @private
+ */
+goog.net.IframeIo.createIframeHtml_ = function(innerFrameName) {
+  var innerFrameNameEscaped = goog.string.htmlEscape(innerFrameName);
+  return goog.html.uncheckedconversions
+      .safeHtmlFromStringKnownToSatisfyTypeContract(
+          goog.string.Const.from(
+              'Short HTML snippet, input escaped, for performance'),
+          '<body><iframe id="' + innerFrameNameEscaped +
+          '" name="' + innerFrameNameEscaped + '"></iframe>');
+};
+
+
+/**
+ * @param {string} innerFrameName
+ * @return {!goog.html.SafeHtml}
+ * @private
+ */
+goog.net.IframeIo.createIframeHtmlWithBaseUri_ = function(innerFrameName) {
+  var innerFrameNameEscaped = goog.string.htmlEscape(innerFrameName);
+  return goog.html.uncheckedconversions
+      .safeHtmlFromStringKnownToSatisfyTypeContract(
+          goog.string.Const.from(
+              'Short HTML snippet, input escaped, safe URL, for performance'),
+          '<head><base href="' +
+          goog.string.htmlEscape(/** @type {string} */ (document.baseURI)) +
+          '"></head>' +
+          '<body><iframe id="' + innerFrameNameEscaped +
+          '" name="' + innerFrameNameEscaped + '"></iframe>');
+};
+
+
 /**
  * Handles the load event of the iframe for IE, determines if the request was
  * successful or not, handles clean up and dispatching of appropriate events.
@@ -1031,7 +1045,7 @@ goog.net.IframeIo.prototype.onIeReadyStateChange_ = function(e) {
       this.handleError_(goog.net.ErrorCode.ACCESS_DENIED);
       return;
     }
-    this.handleLoad_(/** @type {HTMLDocument} */(doc));
+    this.handleLoad_(/** @type {!HTMLDocument} */(doc));
   }
 };
 
@@ -1130,6 +1144,7 @@ goog.net.IframeIo.prototype.handleError_ = function(errorCode,
     this.complete_ = true;
     this.lastErrorCode_ = errorCode;
     if (errorCode == goog.net.ErrorCode.CUSTOM_ERROR) {
+      goog.asserts.assert(goog.isDef(opt_customError));
       this.lastCustomError_ = opt_customError;
     }
     this.dispatchEvent(goog.net.EventType.COMPLETE);
@@ -1160,7 +1175,6 @@ goog.net.IframeIo.prototype.handleIncrementalData_ = function(data) {
  */
 goog.net.IframeIo.prototype.makeReady_ = function() {
   goog.log.info(this.logger_, 'Ready for new requests');
-  var iframe = this.iframe_;
   this.scheduleIframeDisposal_();
   this.disposeForm_();
   this.dispatchEvent(goog.net.EventType.READY);
@@ -1184,8 +1198,9 @@ goog.net.IframeIo.prototype.createIframe_ = function() {
     iframeAttributes.src = 'javascript:""';
   }
 
-  this.iframe_ = /** @type {HTMLIFrameElement} */(
-      goog.dom.getDomHelper(this.form_).createDom('iframe', iframeAttributes));
+  this.iframe_ = /** @type {!HTMLIFrameElement} */(
+      goog.dom.getDomHelper(this.form_).createDom(
+          goog.dom.TagName.IFRAME, iframeAttributes));
 
   var s = this.iframe_.style;
   s.visibility = 'hidden';
@@ -1312,7 +1327,7 @@ goog.net.IframeIo.prototype.disposeForm_ = function() {
  */
 goog.net.IframeIo.prototype.getContentDocument_ = function() {
   if (this.iframe_) {
-    return /** @type {HTMLDocument} */(goog.dom.getFrameContentDocument(
+    return /** @type {!HTMLDocument} */(goog.dom.getFrameContentDocument(
         this.getRequestIframe()));
   }
   return null;
