@@ -429,7 +429,7 @@ function testFieldDescriptor() {
   var descriptor = message.getDescriptor();
   var fields = descriptor.getFields();
 
-  assertEquals(40, fields.length);
+  assertEquals(53, fields.length);
 
   // Check the containing types.
   for (var i = 0; i < fields.length; ++i) {
@@ -480,6 +480,20 @@ function testFieldDescriptor() {
   assertEquals('repeated_int64_number', fields[38].getName());
   assertEquals('repeated_int64_string', fields[39].getName());
 
+  assertEquals('packed_int32', fields[40].getName());
+  assertEquals('packed_int64', fields[41].getName());
+  assertEquals('packed_uint32', fields[42].getName());
+  assertEquals('packed_uint64', fields[43].getName());
+  assertEquals('packed_sint32', fields[44].getName());
+  assertEquals('packed_sint64', fields[45].getName());
+  assertEquals('packed_fixed32', fields[46].getName());
+  assertEquals('packed_fixed64', fields[47].getName());
+  assertEquals('packed_sfixed32', fields[48].getName());
+  assertEquals('packed_sfixed64', fields[49].getName());
+  assertEquals('packed_float', fields[50].getName());
+  assertEquals('packed_double', fields[51].getName());
+  assertEquals('packed_bool', fields[52].getName());
+
   // Check the field types.
   var FieldType = goog.proto2.FieldDescriptor.FieldType;
   assertEquals(FieldType.INT32, fields[0].getFieldType());
@@ -525,6 +539,20 @@ function testFieldDescriptor() {
   assertEquals(FieldType.INT64, fields[38].getFieldType());
   assertEquals(FieldType.INT64, fields[39].getFieldType());
 
+  assertEquals(FieldType.INT32, fields[40].getFieldType());
+  assertEquals(FieldType.INT64, fields[41].getFieldType());
+  assertEquals(FieldType.UINT32, fields[42].getFieldType());
+  assertEquals(FieldType.UINT64, fields[43].getFieldType());
+  assertEquals(FieldType.SINT32, fields[44].getFieldType());
+  assertEquals(FieldType.SINT64, fields[45].getFieldType());
+  assertEquals(FieldType.FIXED32, fields[46].getFieldType());
+  assertEquals(FieldType.FIXED64, fields[47].getFieldType());
+  assertEquals(FieldType.SFIXED32, fields[48].getFieldType());
+  assertEquals(FieldType.SFIXED64, fields[49].getFieldType());
+  assertEquals(FieldType.FLOAT, fields[50].getFieldType());
+  assertEquals(FieldType.DOUBLE, fields[51].getFieldType());
+  assertEquals(FieldType.BOOL, fields[52].getFieldType());
+
   // Check the field native types.
   // Singular.
   assertEquals(Number, fields[0].getNativeType());
@@ -577,6 +605,21 @@ function testFieldDescriptor() {
 
   assertEquals(Number, fields[38].getNativeType());  // [jstype="number"]
   assertEquals(String, fields[39].getNativeType());
+
+  // Packed (only numeric types can be packed).
+  assertEquals(Number, fields[40].getNativeType());
+  assertEquals(Number, fields[41].getNativeType());
+  assertEquals(Number, fields[42].getNativeType());
+  assertEquals(Number, fields[43].getNativeType());
+  assertEquals(Number, fields[44].getNativeType());
+  assertEquals(Number, fields[45].getNativeType());
+  assertEquals(Number, fields[46].getNativeType());
+  assertEquals(Number, fields[47].getNativeType());
+  assertEquals(Number, fields[48].getNativeType());
+  assertEquals(Number, fields[49].getNativeType());
+  assertEquals(Number, fields[50].getNativeType());
+  assertEquals(Number, fields[51].getNativeType());
+  assertEquals(Boolean, fields[52].getNativeType());
 }
 
 function testUnknown() {
