@@ -374,7 +374,7 @@
 
 (schema/defn write-local-cacrl! :- (schema/maybe Exception)
   "Spits the contents of 'cacrl' string to the 'localcacrl' file location if
-  the 'cacrl' string contains valid CRL pem data.  On success, return nil.
+  the 'cacrl' string contains valid CRL pem data. On success, return nil.
   On failure, return the Exception captured from the failed attempt to parse
   the CRL pem data."
   [localcacrl :- schema/Str
@@ -699,7 +699,7 @@
   (set-file-perms (:privatekeydir settings) private-key-dir-perms)
   (-> settings :certdir fs/file ks/mkdirs!)
   (-> settings :requestdir fs/file ks/mkdirs!)
-  (let [ca-cert        (utils/pem->cert (:cacert ca-settings))
+  (let [ca-cert        (utils/pem->ca-cert (:cacert ca-settings) (:cakey ca-settings))
         ca-public-key  (.getPublicKey ca-cert)
         ca-private-key (utils/pem->private-key (:cakey ca-settings))
         next-serial    (next-serial-number! (:serial ca-settings))
@@ -749,51 +749,51 @@
   "Ensure a local copy of the CA cert is available on disk.  cacert is the base
   CA cert file to copy from and localcacert is where the CA cert file should be
   copied to."
-  ([cacert :- schema/Str
-    localcacert :- schema/Str]
-   (if (and (fs/exists? cacert) (not (fs/exists? localcacert)))
-     (do
-       (ks/mkdirs! (fs/parent localcacert))
-       (fs/copy cacert localcacert))
-     (if-not (fs/exists? localcacert)
-       (throw (IllegalStateException.
+  [cacert :- schema/Str
+   localcacert :- schema/Str]
+  (if (and (fs/exists? cacert) (not (fs/exists? localcacert)))
+    (do
+      (ks/mkdirs! (fs/parent localcacert))
+      (fs/copy cacert localcacert))
+    (if-not (fs/exists? localcacert)
+      (throw (IllegalStateException.
                (i18n/trs ":localcacert ({0}) could not be found and no file at :cacert ({1}) to copy it from"
-                    localcacert cacert)))))))
+                    localcacert cacert))))))
 
 (schema/defn ^:always-validate retrieve-ca-crl!
   "Ensure a local copy of the CA CRL, if one exists, is available on disk.
   cacrl is the base CRL file to copy from and localcacrl is where the CRL file
   should be copied to."
-  ([cacrl :- schema/Str
-    localcacrl :- schema/Str]
-   (when (not= cacrl localcacrl)
-     (let [max-attempts 25]
-       (when (fs/exists? cacrl)
-         (ks/mkdirs! (fs/parent localcacrl))
-         ;; Make up to 'max-attempts' tries to copy the cacrl file to the
-         ;; localcacrl file.  The content of the cacrl file is read into memory
-         ;; and parsed for valid CRL pem content during each attempt.  The content
-         ;; in memory is written to the localcacrl file only if it is valid.  This
-         ;; validation is done to protect against a partially written (which could
-         ;; happen if an asynchronous revocation is in process) or corrupt
-         ;; cacrl file being copied.
-         (loop [attempts-left max-attempts]
+  [cacrl :- schema/Str
+   localcacrl :- schema/Str]
+  (when (not= cacrl localcacrl)
+    (let [max-attempts 25]
+      (when (fs/exists? cacrl)
+        (ks/mkdirs! (fs/parent localcacrl))
+        ;; Make up to 'max-attempts' tries to copy the cacrl file to the
+        ;; localcacrl file. The content of the cacrl file is read into memory
+        ;; and parsed for valid CRL pem content during each attempt. The content
+        ;; in memory is written to the localcacrl file only if it is valid. This
+        ;; validation is done to protect against a partially written (which could
+        ;; happen if an asynchronous revocation is in progress) or currupt cacrl
+        ;; file being copied.
+        (loop [attempts-left max-attempts]
            (let [cacrl-as-string (slurp cacrl)]
              (when-let [write-exception (write-local-cacrl!
-                                         localcacrl
-                                         cacrl-as-string)]
+                                          localcacrl
+                                          cacrl-as-string)]
                (if (zero? attempts-left)
                  (log/error (format "%s\n%s\n%s"
                                     (i18n/trs
-                                     "Unable to synchronize crl file {0} to {1}: {2}."
-                                     cacrl localcacrl (.getMessage write-exception))
+                                      "Unable to synchronize crl file {0} to {1}: {2}"
+                                      cacrl localcacrl (.getMessage write-exception))
                                     (i18n/trs
-                                     "Recent changes to the CRL may not have taken effect.")
+                                      "Recent changes to the CRL may not have taken effect.")
                                     (i18n/trs
-                                     "To load the updated CRL reload or restart the Puppet Server service.")))
+                                      "To load the updated CRL, reload or restart the Puppet Server service.")))
                  (do
                    (Thread/sleep 100)
-                   (recur (dec attempts-left))))))))))))
+                   (recur (dec attempts-left)))))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Autosign
@@ -982,7 +982,8 @@
    csr :- CertificateRequest
    {:keys [cacert cakey signeddir ca-ttl serial cert-inventory]} :- CaSettings]
   (let [validity    (cert-validity-dates ca-ttl)
-        cacert      (utils/pem->cert cacert)
+        ;; if part of a CA bundle, the intermediate CA will be first in the chain
+        cacert      (utils/pem->ca-cert cacert cakey)
         signed-cert (utils/sign-certificate (utils/get-subject-from-x509-certificate
                                              cacert)
                                             (utils/pem->private-key cakey)
@@ -1246,7 +1247,7 @@
                     (utils/get-serial))
         new-crl (utils/revoke (utils/pem->crl cacrl)
                               (utils/pem->private-key cakey)
-                              (.getPublicKey (utils/pem->cert cacert))
+                              (.getPublicKey (utils/pem->ca-cert cacert cakey))
                               serial)]
     (utils/crl->pem! new-crl cacrl)
     (log/debug (i18n/trs "Revoked {0} certificate with serial {1}" subject serial))))
