@@ -9,12 +9,9 @@
   "Creates a new JRuby pool context with an empty pool. Once the JRuby
   pool object has been created, it will need to be filled using `prime-pool!`."
   [config :- jruby-schemas/JRubyConfig]
-  (let [agent-shutdown-fn (get-in config [:lifecycle :shutdown-on-error])]
+  (let [shutdown-on-error-fn (get-in config [:lifecycle :shutdown-on-error])]
     {:config config
-     :internal {:pool-agent (jruby-agents/pool-agent agent-shutdown-fn)
-                ;; For an explanation of why we need a separate agent for the `flush-instance`,
-                ;; see the comments in puppetlabs.services.jruby-pool-manager.jruby-agents/send-flush-instance
-                :flush-instance-agent (jruby-agents/pool-agent agent-shutdown-fn)
+     :internal {:modify-instance-agent (jruby-agents/pool-agent shutdown-on-error-fn)
                 :pool-state (atom (jruby-internal/create-pool-from-config config))
                 :event-callbacks (atom [])}}))
 
