@@ -66,7 +66,7 @@ goog.testing.MockClock = function(opt_autoInstall) {
    * right.  For example, the expiration times for each element of the queue
    * might be in the order 300, 200, 200.
    *
-   * @type {Array.<Object>}
+   * @type {Array<Object>}
    * @private
    */
   this.queue_ = [];
@@ -101,7 +101,15 @@ goog.testing.MockClock.REQUEST_ANIMATION_FRAME_TIMEOUT = 20;
 
 
 /**
- * Count of the number of timeouts made.
+ * ID to use for next timeout.  Timeout IDs must never be reused, even across
+ * MockClock instances.
+ * @public {number}
+ */
+goog.testing.MockClock.nextId = Math.round(Math.random() * 10000);
+
+
+/**
+ * Count of the number of timeouts made by this instance.
  * @type {number}
  * @private
  */
@@ -145,12 +153,28 @@ goog.testing.MockClock.prototype.nowMillis_ = 0;
 goog.testing.MockClock.prototype.timeoutDelay_ = 0;
 
 
+/**
+ * The real set timeout for reference.
+ * @const @private {!Function}
+ */
+goog.testing.MockClock.REAL_SETTIMEOUT_ = goog.global.setTimeout;
+
+
 /**
  * Installs the MockClock by overriding the global object's implementation of
  * setTimeout, setInterval, clearTimeout and clearInterval.
  */
 goog.testing.MockClock.prototype.install = function() {
   if (!this.replacer_) {
+    if (goog.testing.MockClock.REAL_SETTIMEOUT_ !== goog.global.setTimeout) {
+      if (typeof console !== 'undefined' && console.warn) {
+        console.warn('Non default setTimeout detected. ' +
+            'Use of multiple MockClock instances or other clock mocking ' +
+            'should be avoided due to unspecified behavior and ' +
+            'the resulting fragility.');
+      }
+    }
+
     var r = this.replacer_ = new goog.testing.PropertyReplacer();
     r.set(goog.global, 'setTimeout', goog.bind(this.setTimeout_, this));
     r.set(goog.global, 'setInterval', goog.bind(this.setInterval_, this));
@@ -163,7 +187,8 @@ goog.testing.MockClock.prototype.install = function() {
     // default to setImmediate, which is replaced above. Note that we test for
     // the presence of goog.async.run.forceNextTick to be resilient to the case
     // where tests replace goog.async.run directly.
-    goog.async.run.forceNextTick && goog.async.run.forceNextTick();
+    goog.async.run.forceNextTick && goog.async.run.forceNextTick(
+        goog.testing.MockClock.REAL_SETTIMEOUT_);
 
     // Replace the requestAnimationFrame functions.
     this.replaceRequestAnimationFrame_();
@@ -187,7 +212,8 @@ goog.testing.MockClock.prototype.replaceRequestAnimationFrame_ = function() {
                       'oRequestAnimationFrame',
                       'msRequestAnimationFrame'];
 
-  var cancelFuncs = ['cancelRequestAnimationFrame',
+  var cancelFuncs = ['cancelAnimationFrame',
+                     'cancelRequestAnimationFrame',
                      'webkitCancelRequestAnimationFrame',
                      'mozCancelRequestAnimationFrame',
                      'oCancelRequestAnimationFrame',
@@ -295,6 +321,43 @@ goog.testing.MockClock.prototype.tick = function(opt_millis) {
 };
 
 
+/**
+ * Takes a promise and then ticks the mock clock. If the promise successfully
+ * resolves, returns the value produced by the promise. If the promise is
+ * rejected, it throws the rejection as an exception. If the promise is not
+ * resolved at all, throws an exception.
+ * Also ticks the general clock by the specified amount.
+ *
+ * @param {!goog.Thenable<T>} promise A promise that should be resolved after
+ *     the mockClock is ticked for the given opt_millis.
+ * @param {number=} opt_millis Number of milliseconds to increment the counter.
+ *     If not specified, clock ticks 1 millisecond.
+ * @return {T}
+ * @template T
+ */
+goog.testing.MockClock.prototype.tickPromise = function(promise, opt_millis) {
+  var value;
+  var error;
+  var resolved = false;
+  promise.then(function(v) {
+    value = v;
+    resolved = true;
+  }, function(e) {
+    error = e;
+    resolved = true;
+  });
+  this.tick(opt_millis);
+  if (!resolved) {
+    throw new Error(
+        'Promise was expected to be resolved after mock clock tick.');
+  }
+  if (error) {
+    throw error;
+  }
+  return value;
+};
+
+
 /**
  * @return {number} The number of timeouts that have been scheduled.
  */
@@ -321,7 +384,9 @@ goog.testing.MockClock.prototype.getCurrentTime = function() {
  *     cleared.
  */
 goog.testing.MockClock.prototype.isTimeoutSet = function(timeoutKey) {
-  return timeoutKey <= this.timeoutsMade_ && !this.deletedKeys_[timeoutKey];
+  return timeoutKey < goog.testing.MockClock.nextId &&
+      timeoutKey >= goog.testing.MockClock.nextId - this.timeoutsMade_ &&
+      !this.deletedKeys_[timeoutKey];
 };
 
 
@@ -392,7 +457,7 @@ goog.testing.MockClock.prototype.scheduleFunction_ = function(
  *
  * @param {Object} timeout The timeout to insert, with numerical runAtMillis
  *     property.
- * @param {Array.<Object>} queue The queue to insert into, with each element
+ * @param {Array<Object>} queue The queue to insert into, with each element
  *     having a numerical runAtMillis property.
  * @private
  */
@@ -434,20 +499,23 @@ goog.testing.MockClock.MAX_INT_ = 2147483647;
  * Schedules a function to be called after {@code millis} milliseconds.
  * Mock implementation for setTimeout.
  * @param {Function} funcToCall The function to call.
- * @param {number} millis The number of milliseconds to call it after.
+ * @param {number=} opt_millis The number of milliseconds to call it after.
  * @return {number} The number of timeouts created.
  * @private
  */
-goog.testing.MockClock.prototype.setTimeout_ = function(funcToCall, millis) {
+goog.testing.MockClock.prototype.setTimeout_ = function(
+    funcToCall, opt_millis) {
+  var millis = opt_millis || 0;
   if (millis > goog.testing.MockClock.MAX_INT_) {
     throw Error(
         'Bad timeout value: ' + millis + '.  Timeouts over MAX_INT ' +
         '(24.8 days) cause timeouts to be fired ' +
         'immediately in most browsers, except for IE.');
   }
-  this.timeoutsMade_ = this.timeoutsMade_ + 1;
-  this.scheduleFunction_(this.timeoutsMade_, funcToCall, millis, false);
-  return this.timeoutsMade_;
+  this.timeoutsMade_++;
+  this.scheduleFunction_(goog.testing.MockClock.nextId, funcToCall, millis,
+      false);
+  return goog.testing.MockClock.nextId++;
 };
 
 
@@ -455,14 +523,17 @@ goog.testing.MockClock.prototype.setTimeout_ = function(funcToCall, millis) {
  * Schedules a function to be called every {@code millis} milliseconds.
  * Mock implementation for setInterval.
  * @param {Function} funcToCall The function to call.
- * @param {number} millis The number of milliseconds between calls.
+ * @param {number=} opt_millis The number of milliseconds between calls.
  * @return {number} The number of timeouts created.
  * @private
  */
-goog.testing.MockClock.prototype.setInterval_ = function(funcToCall, millis) {
-  this.timeoutsMade_ = this.timeoutsMade_ + 1;
-  this.scheduleFunction_(this.timeoutsMade_, funcToCall, millis, true);
-  return this.timeoutsMade_;
+goog.testing.MockClock.prototype.setInterval_ =
+    function(funcToCall, opt_millis) {
+  var millis = opt_millis || 0;
+  this.timeoutsMade_++;
+  this.scheduleFunction_(goog.testing.MockClock.nextId, funcToCall, millis,
+      true);
+  return goog.testing.MockClock.nextId++;
 };
 
 
@@ -517,9 +588,6 @@ goog.testing.MockClock.prototype.clearTimeout_ = function(timeoutKey) {
   // For now, we just hackily fail silently if someone tries to clear a timeout
   // key before we've allocated it.
   // Ideally, we should throw an exception if we see this happening.
-  //
-  // TODO(user): We might also try allocating timeout ids from a global
-  // pool rather than a local pool.
   if (this.isTimeoutSet(timeoutKey)) {
     this.deletedKeys_[timeoutKey] = true;
   }
