@@ -87,8 +87,8 @@ goog.testing.stacktrace.Frame.prototype.toCanonicalString = function() {
 
 /**
  * Maximum number of steps while the call chain is followed.
- * @type {number}
- * @private
+ * @private {number}
+ * @const
  */
 goog.testing.stacktrace.MAX_DEPTH_ = 20;
 
@@ -99,8 +99,8 @@ goog.testing.stacktrace.MAX_DEPTH_ = 20;
  * to exceed Firefox's stack quota. This situation can be encountered
  * when goog.globalEval is invoked with a long argument; such as
  * when loading a module.
- * @type {number}
- * @private
+ * @private {number}
+ * @const
  */
 goog.testing.stacktrace.MAX_FIREFOX_FRAMESTRING_LENGTH_ = 500000;
 
@@ -108,81 +108,95 @@ goog.testing.stacktrace.MAX_FIREFOX_FRAMESTRING_LENGTH_ = 500000;
 /**
  * RegExp pattern for JavaScript identifiers. We don't support Unicode
  * identifiers defined in ECMAScript v3.
- * @type {string}
- * @private
+ * @private {string}
+ * @const
  */
 goog.testing.stacktrace.IDENTIFIER_PATTERN_ = '[a-zA-Z_$][\\w$]*';
 
 
 /**
- * RegExp pattern for function name alias in the Chrome stack trace.
- * @type {string}
- * @private
+ * RegExp pattern for function name alias in the V8 stack trace.
+ * @private {string}
+ * @const
  */
-goog.testing.stacktrace.CHROME_ALIAS_PATTERN_ =
+goog.testing.stacktrace.V8_ALIAS_PATTERN_ =
     '(?: \\[as (' + goog.testing.stacktrace.IDENTIFIER_PATTERN_ + ')\\])?';
 
 
 /**
- * RegExp pattern for function names and constructor calls in the Chrome stack
- * trace.
- * @type {string}
- * @private
+ * RegExp pattern for the context of a function call in a V8 stack trace.
+ * Creates an optional submatch for the namespace identifier including the
+ * "new" keyword for constructor calls (e.g. "new foo.Bar").
+ * @private {string}
+ * @const
  */
-goog.testing.stacktrace.CHROME_FUNCTION_NAME_PATTERN_ =
+goog.testing.stacktrace.V8_CONTEXT_PATTERN_ =
+    '(?:((?:new )?(?:\\[object Object\\]|' +
+    goog.testing.stacktrace.IDENTIFIER_PATTERN_ +
+    '(?:\\.' + goog.testing.stacktrace.IDENTIFIER_PATTERN_ + ')*))\\.)?';
+
+
+/**
+ * RegExp pattern for function names and constructor calls in the V8 stack
+ * trace.
+ * @private {string}
+ * @const
+ */
+goog.testing.stacktrace.V8_FUNCTION_NAME_PATTERN_ =
     '(?:new )?(?:' + goog.testing.stacktrace.IDENTIFIER_PATTERN_ +
     '|<anonymous>)';
 
 
 /**
- * RegExp pattern for function call in the Chrome stack trace.
- * Creates 3 submatches with context object (optional), function name and
- * function alias (optional).
- * @type {string}
- * @private
+ * RegExp pattern for function call in the V8 stack trace. Creates 3 submatches
+ * with context object (optional), function name and function alias (optional).
+ * @private {string}
+ * @const
  */
-goog.testing.stacktrace.CHROME_FUNCTION_CALL_PATTERN_ =
-    ' (?:(.*?)\\.)?(' + goog.testing.stacktrace.CHROME_FUNCTION_NAME_PATTERN_ +
-    ')' + goog.testing.stacktrace.CHROME_ALIAS_PATTERN_;
+goog.testing.stacktrace.V8_FUNCTION_CALL_PATTERN_ =
+    ' ' + goog.testing.stacktrace.V8_CONTEXT_PATTERN_ +
+    '(' + goog.testing.stacktrace.V8_FUNCTION_NAME_PATTERN_ + ')' +
+    goog.testing.stacktrace.V8_ALIAS_PATTERN_;
 
 
 /**
  * RegExp pattern for an URL + position inside the file.
- * @type {string}
- * @private
+ * @private {string}
+ * @const
  */
 goog.testing.stacktrace.URL_PATTERN_ =
     '((?:http|https|file)://[^\\s)]+|javascript:.*)';
 
 
 /**
- * RegExp pattern for an URL + line number + column number in Chrome.
+ * RegExp pattern for an URL + line number + column number in V8.
  * The URL is either in submatch 1 or submatch 2.
- * @type {string}
- * @private
+ * @private {string}
+ * @const
  */
 goog.testing.stacktrace.CHROME_URL_PATTERN_ = ' (?:' +
     '\\(unknown source\\)' + '|' +
     '\\(native\\)' + '|' +
-    '\\((?:eval at )?' + goog.testing.stacktrace.URL_PATTERN_ + '\\)' + '|' +
-    goog.testing.stacktrace.URL_PATTERN_ + ')';
+    '\\((.+)\\)|(.+))';
 
 
 /**
- * Regular expression for parsing one stack frame in Chrome.
- * @type {!RegExp}
- * @private
+ * Regular expression for parsing one stack frame in V8. For more information
+ * on V8 stack frame formats, see
+ * https://code.google.com/p/v8/wiki/JavaScriptStackTraceApi.
+ * @private {!RegExp}
+ * @const
  */
-goog.testing.stacktrace.CHROME_STACK_FRAME_REGEXP_ = new RegExp('^    at' +
-    '(?:' + goog.testing.stacktrace.CHROME_FUNCTION_CALL_PATTERN_ + ')?' +
+goog.testing.stacktrace.V8_STACK_FRAME_REGEXP_ = new RegExp('^    at' +
+    '(?:' + goog.testing.stacktrace.V8_FUNCTION_CALL_PATTERN_ + ')?' +
     goog.testing.stacktrace.CHROME_URL_PATTERN_ + '$');
 
 
 /**
  * RegExp pattern for function call in the Firefox stack trace.
  * Creates 2 submatches with function name (optional) and arguments.
- * @type {string}
- * @private
+ * @private {string}
+ * @const
  */
 goog.testing.stacktrace.FIREFOX_FUNCTION_CALL_PATTERN_ =
     '(' + goog.testing.stacktrace.IDENTIFIER_PATTERN_ + ')?' +
@@ -191,8 +205,8 @@ goog.testing.stacktrace.FIREFOX_FUNCTION_CALL_PATTERN_ =
 
 /**
  * Regular expression for parsing one stack frame in Firefox.
- * @type {!RegExp}
- * @private
+ * @private {!RegExp}
+ * @const
  */
 goog.testing.stacktrace.FIREFOX_STACK_FRAME_REGEXP_ = new RegExp('^' +
     goog.testing.stacktrace.FIREFOX_FUNCTION_CALL_PATTERN_ +
@@ -202,9 +216,8 @@ goog.testing.stacktrace.FIREFOX_STACK_FRAME_REGEXP_ = new RegExp('^' +
 /**
  * RegExp pattern for an anonymous function call in an Opera stack frame.
  * Creates 2 (optional) submatches: the context object and function name.
- * @type {string}
+ * @private {string}
  * @const
- * @private
  */
 goog.testing.stacktrace.OPERA_ANONYMOUS_FUNCTION_NAME_PATTERN_ =
     '<anonymous function(?:\\: ' +
@@ -218,9 +231,8 @@ goog.testing.stacktrace.OPERA_ANONYMOUS_FUNCTION_NAME_PATTERN_ =
  * Creates 4 (optional) submatches: the function name (if not anonymous),
  * the aliased context object and function name (if anonymous), and the
  * function call arguments.
- * @type {string}
+ * @private {string}
  * @const
- * @private
  */
 goog.testing.stacktrace.OPERA_FUNCTION_CALL_PATTERN_ =
     '(?:(?:(' + goog.testing.stacktrace.IDENTIFIER_PATTERN_ + ')|' +
@@ -229,10 +241,11 @@ goog.testing.stacktrace.OPERA_FUNCTION_CALL_PATTERN_ =
 
 
 /**
- * Regular expression for parsing on stack frame in Opera 11.68+
- * @type {!RegExp}
+ * Regular expression for parsing on stack frame in Opera 11.68 - 12.17.
+ * Newer versions of Opera use V8 and stack frames should match against
+ * goog.testing.stacktrace.V8_STACK_FRAME_REGEXP_.
+ * @private {!RegExp}
  * @const
- * @private
  */
 goog.testing.stacktrace.OPERA_STACK_FRAME_REGEXP_ = new RegExp('^' +
     goog.testing.stacktrace.OPERA_FUNCTION_CALL_PATTERN_ +
@@ -241,8 +254,8 @@ goog.testing.stacktrace.OPERA_STACK_FRAME_REGEXP_ = new RegExp('^' +
 
 /**
  * Regular expression for finding the function name in its source.
- * @type {!RegExp}
- * @private
+ * @private {!RegExp}
+ * @const
  */
 goog.testing.stacktrace.FUNCTION_SOURCE_REGEXP_ = new RegExp(
     '^function (' + goog.testing.stacktrace.IDENTIFIER_PATTERN_ + ')');
@@ -251,9 +264,8 @@ goog.testing.stacktrace.FUNCTION_SOURCE_REGEXP_ = new RegExp(
 /**
  * RegExp pattern for function call in a IE stack trace. This expression allows
  * for identifiers like 'Anonymous function', 'eval code', and 'Global code'.
- * @type {string}
+ * @private {string}
  * @const
- * @private
  */
 goog.testing.stacktrace.IE_FUNCTION_CALL_PATTERN_ = '(' +
     goog.testing.stacktrace.IDENTIFIER_PATTERN_ + '(?:\\s+\\w+)*)';
@@ -261,9 +273,8 @@ goog.testing.stacktrace.IE_FUNCTION_CALL_PATTERN_ = '(' +
 
 /**
  * Regular expression for parsing a stack frame in IE.
- * @type {!RegExp}
+ * @private {!RegExp}
  * @const
- * @private
  */
 goog.testing.stacktrace.IE_STACK_FRAME_REGEXP_ = new RegExp('^   at ' +
     goog.testing.stacktrace.IE_FUNCTION_CALL_PATTERN_ +
@@ -274,7 +285,7 @@ goog.testing.stacktrace.IE_STACK_FRAME_REGEXP_ = new RegExp('^   at ' +
 /**
  * Creates a stack trace by following the call chain. Based on
  * {@link goog.debug.getStacktrace}.
- * @return {!Array.<!goog.testing.stacktrace.Frame>} Stack frames.
+ * @return {!Array<!goog.testing.stacktrace.Frame>} Stack frames.
  * @private
  * @suppress {es5Strict}
  */
@@ -338,10 +349,11 @@ goog.testing.stacktrace.followCallChain_ = function() {
  * @private
  */
 goog.testing.stacktrace.parseStackFrame_ = function(frameStr) {
-  var m = frameStr.match(goog.testing.stacktrace.CHROME_STACK_FRAME_REGEXP_);
+  // This match includes newer versions of Opera (15+).
+  var m = frameStr.match(goog.testing.stacktrace.V8_STACK_FRAME_REGEXP_);
   if (m) {
     return new goog.testing.stacktrace.Frame(m[1] || '', m[2] || '', m[3] || '',
-        '', m[4] || m[5] || '');
+        '', m[4] || m[5] || m[6] || '');
   }
 
   if (frameStr.length >
@@ -355,6 +367,7 @@ goog.testing.stacktrace.parseStackFrame_ = function(frameStr) {
         m[3] || '');
   }
 
+  // Match against Presto Opera 11.68 - 12.17.
   m = frameStr.match(goog.testing.stacktrace.OPERA_STACK_FRAME_REGEXP_);
   if (m) {
     return new goog.testing.stacktrace.Frame(m[2] || '', m[1] || m[3] || '',
@@ -446,7 +459,7 @@ goog.testing.stacktrace.htmlEscape_ = function(text) {
 /**
  * Converts the stack frames into canonical format. Chops the beginning and the
  * end of it which come from the testing environment, not from the test itself.
- * @param {!Array.<goog.testing.stacktrace.Frame>} frames The frames.
+ * @param {!Array<goog.testing.stacktrace.Frame>} frames The frames.
  * @return {string} Canonical, pretty printed stack trace.
  * @private
  */
@@ -487,7 +500,7 @@ goog.testing.stacktrace.framesToString_ = function(frames) {
 /**
  * Parses the browser's native stack trace.
  * @param {string} stack Stack trace.
- * @return {!Array.<goog.testing.stacktrace.Frame>} Stack frames. The
+ * @return {!Array<goog.testing.stacktrace.Frame>} Stack frames. The
  *     unrecognized frames will be nulled out.
  * @private
  */
@@ -514,7 +527,7 @@ goog.testing.stacktrace.canonicalize = function(stack) {
 
 /**
  * Returns the native stack trace.
- * @return {string|!Array.<!CallSite>}
+ * @return {string|!Array<!CallSite>}
  * @private
  */
 goog.testing.stacktrace.getNativeStack_ = function() {
@@ -557,8 +570,8 @@ goog.testing.stacktrace.get = function() {
 /**
  * Converts an array of CallSite (elements of a stack trace in V8) to an array
  * of Frames.
- * @param {!Array.<!CallSite>} stack The stack as an array of CallSites.
- * @return {!Array.<!goog.testing.stacktrace.Frame>} The stack as an array of
+ * @param {!Array<!CallSite>} stack The stack as an array of CallSites.
+ * @return {!Array<!goog.testing.stacktrace.Frame>} The stack as an array of
  *     Frames.
  * @private
  */
