@@ -60,7 +60,7 @@ goog.ui.ac.AutoComplete = function(matcher, renderer, selectionHandler) {
   /**
    * A data-source which provides autocomplete suggestions.
    *
-   * TODO(user): Tighten the type to !goog.ui.ac.AutoComplete.Matcher.
+   * TODO(chrishenry): Tighten the type to !goog.ui.ac.AutoComplete.Matcher.
    *
    * @type {Object}
    * @protected
@@ -72,7 +72,7 @@ goog.ui.ac.AutoComplete = function(matcher, renderer, selectionHandler) {
    * A handler which interacts with the input DOM element (textfield, textarea,
    * or richedit).
    *
-   * TODO(user): Tighten the type to !Object.
+   * TODO(chrishenry): Tighten the type to !Object.
    *
    * @type {Object}
    * @protected
@@ -107,7 +107,7 @@ goog.ui.ac.AutoComplete = function(matcher, renderer, selectionHandler) {
 
   /**
    * Autocomplete suggestion items.
-   * @type {Array}
+   * @type {Array<?>}
    * @protected
    * @suppress {underscore|visibility}
    */
@@ -125,7 +125,7 @@ goog.ui.ac.AutoComplete = function(matcher, renderer, selectionHandler) {
    * Id of the first row in autocomplete menu. Note that new ids are assigned
    * everytime new suggestions are fetched.
    *
-   * TODO(user): Figure out what subclass does with this value
+   * TODO(chrishenry): Figure out what subclass does with this value
    * and whether we should expose a more proper API.
    *
    * @type {number}
@@ -153,7 +153,7 @@ goog.ui.ac.AutoComplete = function(matcher, renderer, selectionHandler) {
    * Mapping from text input element to the anchor element. If the
    * mapping does not exist, the input element will act as the anchor
    * element.
-   * @type {Object.<Element>}
+   * @type {Object<Element>}
    * @private
    */
   this.inputToAnchorMap_ = {};
@@ -323,7 +323,7 @@ goog.ui.ac.AutoComplete.prototype.getToken = function() {
 /**
  * Sets the current token (without changing the rendered autocompletion).
  *
- * NOTE(user): This method will likely go away when we figure
+ * NOTE(chrishenry): This method will likely go away when we figure
  * out a better API.
  *
  * @param {?string} token The new token.
@@ -346,7 +346,7 @@ goog.ui.ac.AutoComplete.prototype.getSuggestion = function(index) {
 
 
 /**
- * @return {!Array} The current autocomplete suggestion items.
+ * @return {!Array<?>} The current autocomplete suggestion items.
  */
 goog.ui.ac.AutoComplete.prototype.getAllSuggestions = function() {
   return goog.asserts.assert(this.rows_);
@@ -369,20 +369,6 @@ goog.ui.ac.AutoComplete.prototype.getHighlightedId = function() {
 };
 
 
-/**
- * Sets the current highlighted row to the given id (not index). Note
- * that this does not change any rendering.
- *
- * NOTE(user): This method will likely go away when we figure
- * out a better API.
- *
- * @param {number} id The new highlighted row id.
- */
-goog.ui.ac.AutoComplete.prototype.setHighlightedIdInternal = function(id) {
-  this.hiliteId_ = id;
-};
-
-
 /**
  * Generic event handler that handles any events this object is listening to.
  * @param {goog.events.Event} e Event Object.
@@ -777,7 +763,7 @@ goog.ui.ac.AutoComplete.prototype.disposeInternal = function() {
  * </pre>
  *
  * @param {string} matchedToken Token that corresponds with the rows.
- * @param {!Array} rows Set of data that match the given token.
+ * @param {!Array<?>} rows Set of data that match the given token.
  * @param {(boolean|goog.ui.ac.RenderOptions)=} opt_options If true,
  *     keeps the currently hilited (by index) element hilited. If false not.
  *     Otherwise a RenderOptions object.
@@ -798,7 +784,7 @@ goog.ui.ac.AutoComplete.prototype.matchListener_ =
 
 /**
  * Renders the rows and adds highlighting.
- * @param {!Array} rows Set of data that match the given token.
+ * @param {!Array<?>} rows Set of data that match the given token.
  * @param {(boolean|goog.ui.ac.RenderOptions)=} opt_options If true,
  *     keeps the currently hilited (by index) element hilited. If false not.
  *     Otherwise a RenderOptions object.
