@@ -24,6 +24,9 @@ goog.provide('goog.ui.KeyboardShortcutHandler');
 goog.provide('goog.ui.KeyboardShortcutHandler.EventType');
 
 goog.require('goog.Timer');
+goog.require('goog.array');
+goog.require('goog.asserts');
+goog.require('goog.dom.TagName');
 goog.require('goog.events');
 goog.require('goog.events.Event');
 goog.require('goog.events.EventTarget');
@@ -57,22 +60,28 @@ goog.ui.KeyboardShortcutHandler = function(keyTarget) {
    * Registered keyboard shortcuts tree. Stored as a map with the keyCode and
    * modifier(s) as the key and either a list of further strokes or the shortcut
    * task identifier as the value.
-   * @type {Object}
-   * @see #makeKey_
+   * @type {!goog.ui.KeyboardShortcutHandler.SequenceTree_}
+   * @see #makeStroke_
    * @private
    */
   this.shortcuts_ = {};
 
   /**
-   * List of the last sequence of strokes. Object contains time last key was
-   * pressed and an array of strokes, represented by numeric value.
-   * @type {Object}
+   * The currently active shortcut sequence tree, which represents the position
+   * in the complete shortcuts_ tree reached by recent key strokes.
+   * @type {!goog.ui.KeyboardShortcutHandler.SequenceTree_}
    * @private
    */
-  this.lastKeys_ = {
-    strokes: [],
-    time: 0
-  };
+  this.currentTree_ = this.shortcuts_;
+
+  /**
+   * The time (in ms, epoch time) of the last keystroke which made progress in
+   * the shortcut sequence tree (i.e. the time that currentTree_ was last set).
+   * Used for timing out stroke sequences.
+   * @type {number}
+   * @private
+   */
+  this.lastStrokeTime_ = 0;
 
   /**
    * List of numeric key codes for keys that are safe to always regarded as
@@ -143,6 +152,61 @@ goog.inherits(goog.ui.KeyboardShortcutHandler, goog.events.EventTarget);
 goog.tagUnsealableClass(goog.ui.KeyboardShortcutHandler);
 
 
+
+/**
+ * A node in a keyboard shortcut sequence tree. A node is either:
+ * 1. A terminal node with a non-nullable shortcut string which is the
+ *    identifier for the shortcut triggered by traversing the tree to that node.
+ * 2. An internal node with a null shortcut string and a
+ *    {@code goog.ui.KeyboardShortcutHandler.SequenceTree_} representing the
+ *    continued stroke sequences from this node.
+ * For clarity, the static factory methods for creating internal and terminal
+ * nodes below should be used rather than using this constructor directly.
+ * @param {string=} opt_shortcut The shortcut identifier, for terminal nodes.
+ * @constructor
+ * @struct
+ * @private
+ */
+goog.ui.KeyboardShortcutHandler.SequenceNode_ = function(opt_shortcut) {
+  /** @const {?string} The shorcut action identifier, for terminal nodes. */
+  this.shortcut = opt_shortcut || null;
+
+  /** @const {goog.ui.KeyboardShortcutHandler.SequenceTree_} */
+  this.next = opt_shortcut ? null : {};
+};
+
+
+/**
+ * Creates a terminal shortcut sequence node for the given shortcut identifier.
+ * @param {string} shortcut The shortcut identifier.
+ * @return {!goog.ui.KeyboardShortcutHandler.SequenceNode_}
+ * @private
+ */
+goog.ui.KeyboardShortcutHandler.createTerminalNode_ = function(shortcut) {
+  return new goog.ui.KeyboardShortcutHandler.SequenceNode_(shortcut);
+};
+
+
+/**
+ * Creates an internal shortcut sequence node - a non-terminal part of a
+ * keyboard sequence.
+ * @return {!goog.ui.KeyboardShortcutHandler.SequenceNode_}
+ * @private
+ */
+goog.ui.KeyboardShortcutHandler.createInternalNode_ = function() {
+  return new goog.ui.KeyboardShortcutHandler.SequenceNode_();
+};
+
+
+/**
+ * A map of strokes (represented as numbers) to the nodes reached by those
+ * strokes.
+ * @typedef {Object<number, goog.ui.KeyboardShortcutHandler.SequenceNode_>}
+ * @private
+ */
+goog.ui.KeyboardShortcutHandler.SequenceTree_;
+
+
 /**
  * Maximum allowed delay, in milliseconds, allowed between the first and second
  * key in a key sequence.
@@ -166,7 +230,7 @@ goog.ui.KeyboardShortcutHandler.Modifiers = {
 
 /**
  * Keys marked as global by default.
- * @type {Array.<goog.events.KeyCodes>}
+ * @type {Array<goog.events.KeyCodes>}
  * @private
  */
 goog.ui.KeyboardShortcutHandler.DEFAULT_GLOBAL_KEYS_ = [
@@ -190,7 +254,7 @@ goog.ui.KeyboardShortcutHandler.DEFAULT_GLOBAL_KEYS_ = [
 /**
  * Text input types to allow only ENTER shortcuts.
  * Web Forms 2.0 for HTML5: Section 4.10.7 from 29 May 2012.
- * @type {Array.<string>}
+ * @type {Array<string>}
  * @private
  */
 goog.ui.KeyboardShortcutHandler.DEFAULT_TEXT_INPUTS_ = [
@@ -223,7 +287,7 @@ goog.ui.KeyboardShortcutHandler.EventType = {
 
 /**
  * Cache for name to key code lookup.
- * @type {Object}
+ * @type {Object<number>}
  * @private
  */
 goog.ui.KeyboardShortcutHandler.nameToKeyCodeCache_;
@@ -274,7 +338,9 @@ goog.ui.KeyboardShortcutHandler.getKeyCode = function(name) {
   if (!goog.ui.KeyboardShortcutHandler.nameToKeyCodeCache_) {
     var map = {};
     for (var key in goog.events.KeyNames) {
-      map[goog.events.KeyNames[key]] = key;
+      // Explicitly convert the stringified map keys to numbers and normalize.
+      map[goog.events.KeyNames[key]] =
+          goog.events.KeyCodes.normalizeKeyCode(parseInt(key, 10));
     }
     goog.ui.KeyboardShortcutHandler.nameToKeyCodeCache_ = map;
   }
@@ -404,7 +470,7 @@ goog.ui.KeyboardShortcutHandler.prototype.setAllowSpaceKeyOnButtons = function(
  * @param {string} identifier Identifier for the task performed by the keyboard
  *                 combination. Multiple shortcuts can be provided for the same
  *                 task by specifying the same identifier.
- * @param {...(number|string|Array.<number>)} var_args See below.
+ * @param {...(number|string|Array<number>)} var_args See below.
  *
  * param {number} keyCode Numeric code for key
  * param {number=} opt_modifiers Bitmap indicating required modifier keys.
@@ -460,14 +526,14 @@ goog.ui.KeyboardShortcutHandler.prototype.registerShortcut = function(
  * {@link #registerShortcut} for syntax. In that case the method only takes one
  * argument.
  *
- * @param {...(number|string|Array.<number>)} var_args String representation, or
+ * @param {...(number|string|Array<number>)} var_args String representation, or
  *     array or list of alternating key codes and modifiers.
  */
 goog.ui.KeyboardShortcutHandler.prototype.unregisterShortcut = function(
     var_args) {
-  // Remove shortcut from tree
-  goog.ui.KeyboardShortcutHandler.setShortcut_(
-      this.shortcuts_, this.interpretStrokes_(0, arguments), null);
+  // Remove shortcut from tree.
+  goog.ui.KeyboardShortcutHandler.unsetShortcut_(
+      this.shortcuts_, this.interpretStrokes_(0, arguments));
 };
 
 
@@ -487,7 +553,7 @@ goog.ui.KeyboardShortcutHandler.prototype.unregisterShortcut = function(
  * {@link #registerShortcut} for syntax. In that case the method only takes one
  * argument.
  *
- * @param {...(number|string|Array.<number>)} var_args String representation, or
+ * @param {...(number|string|Array<number>)} var_args String representation, or
  *     array or list of alternating key codes and modifiers.
  * @return {boolean} Whether the specified keyboard shortcut is registered.
  */
@@ -504,8 +570,7 @@ goog.ui.KeyboardShortcutHandler.prototype.isShortcutRegistered = function(
  * @param {Object} args The "arguments" array passed
  *     to registerShortcut or unregisterShortcut.  Please see the comments in
  *     registerShortcut for list of allowed forms.
- * @return {!Array.<Object>} The sequence of objects containing the
- *     keyCode and modifiers of each key in sequence.
+ * @return {!Array<number>} The sequence of strokes, represented as numbers.
  * @private
  */
 goog.ui.KeyboardShortcutHandler.prototype.interpretStrokes_ = function(
@@ -514,8 +579,14 @@ goog.ui.KeyboardShortcutHandler.prototype.interpretStrokes_ = function(
 
   // Build strokes array from string.
   if (goog.isString(args[initialIndex])) {
-    strokes = goog.ui.KeyboardShortcutHandler.parseStringShortcut(
-        args[initialIndex]);
+    strokes = goog.array.map(
+        goog.ui.KeyboardShortcutHandler.parseStringShortcut(args[initialIndex]),
+        function(stroke) {
+          goog.asserts.assertNumber(
+              stroke.keyCode, 'A non-modifier key is needed in each stroke.');
+          return goog.ui.KeyboardShortcutHandler.makeStroke_(
+              stroke.keyCode, stroke.modifiers);
+        });
 
   // Build strokes array from arguments list or from array.
   } else {
@@ -527,10 +598,8 @@ goog.ui.KeyboardShortcutHandler.prototype.interpretStrokes_ = function(
 
     strokes = [];
     for (; i < strokesArgs.length; i += 2) {
-      strokes.push({
-        keyCode: strokesArgs[i],
-        modifiers: strokesArgs[i + 1]
-      });
+      strokes.push(goog.ui.KeyboardShortcutHandler.makeStroke_(
+          strokesArgs[i], strokesArgs[i + 1]));
     }
   }
 
@@ -549,7 +618,7 @@ goog.ui.KeyboardShortcutHandler.prototype.unregisterAll = function() {
 /**
  * Sets the global keys; keys that are safe to always regarded as shortcuts,
  * even if entered in a textarea or input field.
- * @param {Array.<number>} keys List of keys.
+ * @param {Array<number>} keys List of keys.
  */
 goog.ui.KeyboardShortcutHandler.prototype.setGlobalKeys = function(keys) {
   this.globalKeys_ = goog.object.createSet(keys);
@@ -557,7 +626,7 @@ goog.ui.KeyboardShortcutHandler.prototype.setGlobalKeys = function(keys) {
 
 
 /**
- * @return {!Array.<string>} The global keys, i.e. keys that are safe to always
+ * @return {!Array<string>} The global keys, i.e. keys that are safe to always
  *     regard as shortcuts, even if entered in a textarea or input field.
  */
 goog.ui.KeyboardShortcutHandler.prototype.getGlobalKeys = function() {
@@ -588,7 +657,8 @@ goog.ui.KeyboardShortcutHandler.prototype.getEventType =
 /**
  * Builds stroke array from string representation of shortcut.
  * @param {string} s String representation of shortcut.
- * @return {!Array.<Object>} The stroke array.
+ * @return {!Array<!{keyCode: ?number, modifiers: number}>} The stroke array.  A
+ *     null keyCode means no non-modifier key was part of the stroke.
  */
 goog.ui.KeyboardShortcutHandler.parseStringShortcut = function(s) {
   // Normalize whitespace and force to lower case.
@@ -600,7 +670,9 @@ goog.ui.KeyboardShortcutHandler.parseStringShortcut = function(s) {
   var strokes = [];
   for (var group, i = 0; group = groups[i]; i++) {
     var keys = group.split('+');
-    var keyCode, modifiers = goog.ui.KeyboardShortcutHandler.Modifiers.NONE;
+    // Explicitly re-initialize key data (JS does not have block scoping).
+    var keyCode = null;
+    var modifiers = goog.ui.KeyboardShortcutHandler.Modifiers.NONE;
     for (var key, j = 0; key = keys[j]; j++) {
       switch (key) {
         case 'shift':
@@ -616,7 +688,12 @@ goog.ui.KeyboardShortcutHandler.parseStringShortcut = function(s) {
           modifiers |= goog.ui.KeyboardShortcutHandler.Modifiers.META;
           continue;
       }
+      if (!goog.isNull(keyCode)) {
+        goog.asserts.fail('At most one non-modifier key can be in a stroke.');
+      }
       keyCode = goog.ui.KeyboardShortcutHandler.getKeyCode(key);
+      goog.asserts.assertNumber(
+          keyCode, 'Key name not found in goog.events.KeyNames: ' + key);
       break;
     }
     strokes.push({keyCode: keyCode, modifiers: modifiers});
@@ -772,78 +849,96 @@ goog.ui.KeyboardShortcutHandler.prototype.clearKeyListener = function() {
 
 
 /**
- * Adds or removes a stroke node to/from the given parent node.
- * @param {Object} parent Parent node to add/remove stroke to/from.
- * @param {Array.<Object>} strokes Array of strokes for shortcut.
- * @param {?string} identifier Identifier for the task performed by shortcut or
- *     null to clear.
+ * Adds a shortcut stroke sequence to the given sequence tree. Recursive.
+ * @param {!goog.ui.KeyboardShortcutHandler.SequenceTree_} tree The stroke
+ *     sequence tree to add to.
+ * @param {Array<number>} strokes Array of strokes for shortcut.
+ * @param {string} identifier Identifier for the task performed by shortcut.
  * @private
  */
-goog.ui.KeyboardShortcutHandler.setShortcut_ = function(parent,
-                                                        strokes,
-                                                        identifier) {
+goog.ui.KeyboardShortcutHandler.setShortcut_ = function(
+    tree, strokes, identifier) {
   var stroke = strokes.shift();
-  var key = goog.ui.KeyboardShortcutHandler.makeKey_(stroke.keyCode,
-                                                     stroke.modifiers);
-  var node = parent[key];
-  if (node && identifier && (strokes.length == 0 || goog.isString(node))) {
+  var node = tree[stroke];
+  if (node && (strokes.length == 0 || node.shortcut)) {
+    // This new shortcut would override an existing shortcut or shortcut prefix
+    // (since the new strokes end at an existing node), or an existing shortcut
+    // would be triggered by the prefix to this new shortcut (since there is
+    // already a terminal node on the path we are trying to create).
     throw Error('Keyboard shortcut conflicts with existing shortcut');
   }
 
   if (strokes.length) {
-    if (!node) {
-      node = parent[key] = {};
-    }
-    goog.ui.KeyboardShortcutHandler.setShortcut_(node,
-                                                 strokes,
-                                                 identifier);
-  }
-  else {
-    parent[key] = identifier;
+    node = goog.object.setIfUndefined(tree, stroke.toString(),
+        goog.ui.KeyboardShortcutHandler.createInternalNode_());
+    goog.ui.KeyboardShortcutHandler.setShortcut_(
+        goog.asserts.assert(node.next, 'An internal node must have a next map'),
+        strokes, identifier);
+  } else {
+    // Add a terminal node.
+    tree[stroke] =
+        goog.ui.KeyboardShortcutHandler.createTerminalNode_(identifier);
   }
 };
 
 
 /**
- * Returns shortcut for a specific set of strokes.
- * @param {Array.<number>} strokes Strokes array.
- * @param {number=} opt_index Index in array to start with.
- * @param {Object=} opt_list List to search for shortcut in.
- * @return {string|Object} The shortcut.
+ * Removes a shortcut stroke sequence from the given sequence tree, pruning any
+ * dead branches of the tree. Recursive.
+ * @param {!goog.ui.KeyboardShortcutHandler.SequenceTree_} tree The stroke
+ *     sequence tree to remove from.
+ * @param {Array<number>} strokes Array of strokes for shortcut to remove.
  * @private
  */
-goog.ui.KeyboardShortcutHandler.prototype.getShortcut_ = function(
-    strokes, opt_index, opt_list) {
+goog.ui.KeyboardShortcutHandler.unsetShortcut_ = function(tree, strokes) {
+  var stroke = strokes.shift();
+  var node = tree[stroke];
 
-  var list = opt_list || this.shortcuts_;
-  var index = opt_index || 0;
-  var stroke = strokes[index];
-  var node = list[stroke];
-
-  if (node && !goog.isString(node) && strokes.length - index > 1) {
-    return this.getShortcut_(strokes, index + 1, node);
+  if (!node) {
+    // The given stroke sequence is not in the tree.
+    return;
+  }
+  if (strokes.length == 0) {
+    // Base case - the end of the stroke sequence.
+    if (!node.shortcut) {
+      // The given stroke sequence does not end at a terminal node.
+      return;
+    }
+    delete tree[stroke];
+  } else {
+    if (!node.next) {
+      // The given stroke sequence is not in the tree.
+      return;
+    }
+    // Recursively remove the rest of the shortcut sequence from the node.next
+    // subtree.
+    goog.ui.KeyboardShortcutHandler.unsetShortcut_(node.next, strokes);
+    if (goog.object.isEmpty(node.next)) {
+      // The node.next subtree is now empty (the last stroke in it was just
+      // removed), so prune this dead branch of the tree.
+      delete tree[stroke];
+    }
   }
-
-  return node;
 };
 
 
 /**
  * Checks if a particular keyboard shortcut is registered.
- * @param {Array.<Object>} strokes Strokes array.
+ * @param {Array<number>} strokes Strokes array.
  * @return {boolean} True iff the keyboard is registred.
  * @private
  */
 goog.ui.KeyboardShortcutHandler.prototype.checkShortcut_ = function(strokes) {
-  var node = this.shortcuts_;
-  while (strokes.length > 0 && node) {
-    var stroke = strokes.shift();
-    var key = goog.ui.KeyboardShortcutHandler.makeKey_(stroke.keyCode,
-                                                       stroke.modifiers);
-    node = node[key];
-    if (goog.isString(node)) {
+  var tree = this.shortcuts_;
+  while (strokes.length > 0 && tree) {
+    var node = tree[strokes.shift()];
+    if (!node) {
+      return false;
+    }
+    if (strokes.length == 0 && node.shortcut) {
       return true;
     }
+    tree = node.next;
   }
   return false;
 };
@@ -860,7 +955,7 @@ goog.ui.KeyboardShortcutHandler.prototype.checkShortcut_ = function(strokes) {
  * @return {number} The key.
  * @private
  */
-goog.ui.KeyboardShortcutHandler.makeKey_ = function(keyCode, modifiers) {
+goog.ui.KeyboardShortcutHandler.makeStroke_ = function(keyCode, modifiers) {
   // Make sure key code is just 8 bits and OR it with the modifiers left shifted
   // 8 bits.
   return (keyCode & 255) | (modifiers << 8);
@@ -885,94 +980,75 @@ goog.ui.KeyboardShortcutHandler.prototype.handleKeyDown_ = function(event) {
     return;
   }
 
-  var keyCode = goog.userAgent.GECKO ?
-      goog.events.KeyCodes.normalizeGeckoKeyCode(event.keyCode) :
-      event.keyCode;
+  var keyCode = goog.events.KeyCodes.normalizeKeyCode(event.keyCode);
 
   var modifiers =
       (event.shiftKey ? goog.ui.KeyboardShortcutHandler.Modifiers.SHIFT : 0) |
       (event.ctrlKey ? goog.ui.KeyboardShortcutHandler.Modifiers.CTRL : 0) |
       (event.altKey ? goog.ui.KeyboardShortcutHandler.Modifiers.ALT : 0) |
       (event.metaKey ? goog.ui.KeyboardShortcutHandler.Modifiers.META : 0);
-  var stroke = goog.ui.KeyboardShortcutHandler.makeKey_(keyCode, modifiers);
+  var stroke = goog.ui.KeyboardShortcutHandler.makeStroke_(keyCode, modifiers);
 
-  // Check if any previous strokes where entered within the acceptable time
-  // period.
-  var node, shortcut;
-  var now = goog.now();
-  if (this.lastKeys_.strokes.length && now - this.lastKeys_.time <=
-      goog.ui.KeyboardShortcutHandler.MAX_KEY_SEQUENCE_DELAY) {
-    node = this.getShortcut_(this.lastKeys_.strokes);
-  } else {
-    this.lastKeys_.strokes.length = 0;
+  if (!this.currentTree_[stroke] || this.hasSequenceTimedOut_()) {
+    // Either this stroke does not continue any active sequence, or the
+    // currently active sequence has timed out. Reset shortcut tree progress.
+    this.setCurrentTree_(this.shortcuts_);
   }
 
-  // Check if this stroke triggers a shortcut, either on its own or combined
-  // with previous strokes.
-  node = node ? node[stroke] : this.shortcuts_[stroke];
+  var node = this.currentTree_[stroke];
   if (!node) {
-    node = this.shortcuts_[stroke];
-    this.lastKeys_.strokes = [];
+    // This stroke does not correspond to a shortcut or continued sequence.
+    return;
   }
-  // Check if stroke triggers a node.
-  if (node && goog.isString(node)) {
-    shortcut = node;
+  if (node.next) {
+    // This stroke does not trigger a shortcut, but entered stroke(s) are a part
+    // of a sequence. Progress in the sequence tree and record time to allow the
+    // following stroke(s) to trigger the shortcut.
+    this.setCurrentTree_(node.next);
+    // Prevent default action so that the rest of the stroke sequence can be
+    // completed.
+    event.preventDefault();
+    return;
+  }
+  // This stroke triggers a shortcut. Any active sequence has been completed, so
+  // reset the sequence tree.
+  this.setCurrentTree_(this.shortcuts_);
+
+  // Dispatch the triggered keyboard shortcut event. In addition to the generic
+  // keyboard shortcut event a more specific fine grained one, specific for the
+  // shortcut identifier, is fired.
+  if (this.alwaysPreventDefault_) {
+    event.preventDefault();
   }
 
-  // Entered stroke(s) are a part of a sequence, store stroke and record
-  // time to allow the following stroke(s) to trigger the shortcut.
-  else if (node) {
-    this.lastKeys_.strokes.push(stroke);
-    this.lastKeys_.time = now;
-    // Prevent default action so find-as-you-type doesn't steal keyboard focus.
-    if (goog.userAgent.GECKO) {
-      event.preventDefault();
-    }
+  if (this.alwaysStopPropagation_) {
+    event.stopPropagation();
   }
 
-  // No strokes for sequence, clear stored strokes.
-  else {
-    this.lastKeys_.strokes.length = 0;
+  var shortcut = goog.asserts.assertString(
+      node.shortcut, 'A terminal node must have a string shortcut identifier.');
+  // Dispatch SHORTCUT_TRIGGERED event
+  var target = /** @type {Node} */ (event.target);
+  var triggerEvent = new goog.ui.KeyboardShortcutEvent(
+      goog.ui.KeyboardShortcutHandler.EventType.SHORTCUT_TRIGGERED, shortcut,
+      target);
+  var retVal = this.dispatchEvent(triggerEvent);
+
+  // Dispatch SHORTCUT_PREFIX_<identifier> event
+  var prefixEvent = new goog.ui.KeyboardShortcutEvent(
+      goog.ui.KeyboardShortcutHandler.EventType.SHORTCUT_PREFIX + shortcut,
+      shortcut, target);
+  retVal &= this.dispatchEvent(prefixEvent);
+
+  // The default action is prevented if 'preventDefault' was
+  // called on either event, or if a listener returned false.
+  if (!retVal) {
+    event.preventDefault();
   }
 
-  // Dispatch keyboard shortcut event if a shortcut was triggered. In addition
-  // to the generic keyboard shortcut event a more specific fine grained one,
-  // specific for the shortcut identifier, is fired.
-  if (shortcut) {
-    if (this.alwaysPreventDefault_) {
-      event.preventDefault();
-    }
-
-    if (this.alwaysStopPropagation_) {
-      event.stopPropagation();
-    }
-
-    var types = goog.ui.KeyboardShortcutHandler.EventType;
-
-    // Dispatch SHORTCUT_TRIGGERED event
-    var target = /** @type {Node} */ (event.target);
-    var triggerEvent = new goog.ui.KeyboardShortcutEvent(
-        types.SHORTCUT_TRIGGERED, shortcut, target);
-    var retVal = this.dispatchEvent(triggerEvent);
-
-    // Dispatch SHORTCUT_PREFIX_<identifier> event
-    var prefixEvent = new goog.ui.KeyboardShortcutEvent(
-        types.SHORTCUT_PREFIX + shortcut, shortcut, target);
-    retVal &= this.dispatchEvent(prefixEvent);
-
-    // The default action is prevented if 'preventDefault' was
-    // called on either event, or if a listener returned false.
-    if (!retVal) {
-      event.preventDefault();
-    }
-
-    // Clear stored strokes
-    this.lastKeys_.strokes.length = 0;
-
-    // For Firefox, track which shortcut key was pushed.
-    if (goog.userAgent.GECKO) {
-      this.activeShortcutKeyForGecko_ = keyCode;
-    }
+  // For Firefox, track which shortcut key was pushed.
+  if (goog.userAgent.GECKO) {
+    this.activeShortcutKeyForGecko_ = keyCode;
   }
 };
 
@@ -994,8 +1070,10 @@ goog.ui.KeyboardShortcutHandler.prototype.isValidShortcut_ = function(event) {
   }
   var el = /** @type {Element} */ (event.target);
   var isFormElement =
-      el.tagName == 'TEXTAREA' || el.tagName == 'INPUT' ||
-      el.tagName == 'BUTTON' || el.tagName == 'SELECT';
+      el.tagName == goog.dom.TagName.TEXTAREA ||
+      el.tagName == goog.dom.TagName.INPUT ||
+      el.tagName == goog.dom.TagName.BUTTON ||
+      el.tagName == goog.dom.TagName.SELECT;
 
   var isContentEditable = !isFormElement && (el.isContentEditable ||
       (el.ownerDocument && el.ownerDocument.designMode == 'on'));
@@ -1020,11 +1098,12 @@ goog.ui.KeyboardShortcutHandler.prototype.isValidShortcut_ = function(event) {
     return true;
   }
   // Allow ENTER to be used as shortcut for text inputs.
-  if (el.tagName == 'INPUT' && this.textInputs_[el.type]) {
+  if (el.tagName == goog.dom.TagName.INPUT && this.textInputs_[el.type]) {
     return keyCode == goog.events.KeyCodes.ENTER;
   }
   // Checkboxes, radiobuttons and buttons. Allow all but SPACE as shortcut.
-  if (el.tagName == 'INPUT' || el.tagName == 'BUTTON') {
+  if (el.tagName == goog.dom.TagName.INPUT ||
+      el.tagName == goog.dom.TagName.BUTTON) {
     // TODO(gboyer): If more flexibility is needed, create protected helper
     // methods for each case (e.g. button, input, etc).
     if (this.allowSpaceKeyOnButtons_) {
@@ -1038,6 +1117,28 @@ goog.ui.KeyboardShortcutHandler.prototype.isValidShortcut_ = function(event) {
 };
 
 
+/**
+ * @return {boolean} True iff the current stroke sequence has timed out.
+ * @private
+ */
+goog.ui.KeyboardShortcutHandler.prototype.hasSequenceTimedOut_ = function() {
+  return goog.now() - this.lastStrokeTime_ >=
+      goog.ui.KeyboardShortcutHandler.MAX_KEY_SEQUENCE_DELAY;
+};
+
+
+/**
+ * Sets the current keyboard shortcut sequence tree and updates the last stroke
+ * time.
+ * @param {!goog.ui.KeyboardShortcutHandler.SequenceTree_} tree
+ * @private
+ */
+goog.ui.KeyboardShortcutHandler.prototype.setCurrentTree_ = function(tree) {
+  this.currentTree_ = tree;
+  this.lastStrokeTime_ = goog.now();
+};
+
+
 
 /**
  * Object representing a keyboard shortcut event.
