@@ -39,7 +39,10 @@
 
 goog.provide('goog.ui.PopupMenu');
 
+goog.require('goog.events');
+goog.require('goog.events.BrowserEvent');
 goog.require('goog.events.EventType');
+goog.require('goog.events.KeyCodes');
 goog.require('goog.positioning.AnchoredViewportPosition');
 goog.require('goog.positioning.Corner');
 goog.require('goog.positioning.MenuAnchoredPosition');
@@ -56,8 +59,8 @@ goog.require('goog.userAgent');
 
 /**
  * A basic menu class.
- * @param {goog.dom.DomHelper=} opt_domHelper Optional DOM helper.
- * @param {goog.ui.MenuRenderer=} opt_renderer Renderer used to render or
+ * @param {?goog.dom.DomHelper=} opt_domHelper Optional DOM helper.
+ * @param {?goog.ui.MenuRenderer=} opt_renderer Renderer used to render or
  *     decorate the container; defaults to {@link goog.ui.MenuRenderer}.
  * @extends {goog.ui.Menu}
  * @constructor
@@ -99,7 +102,7 @@ goog.ui.PopupMenu.prototype.lastHide_ = 0;
 
 /**
  * Current element where the popup menu is anchored.
- * @type {Element}
+ * @type {?Element}
  * @private
  */
 goog.ui.PopupMenu.prototype.currentAnchor_ = null;
@@ -109,7 +112,7 @@ goog.ui.PopupMenu.prototype.currentAnchor_ = null;
  * Decorate an existing HTML structure with the menu. Menu items will be
  * constructed from elements with classname 'goog-menuitem', separators will be
  * made from HR elements.
- * @param {Element} element Element to decorate.
+ * @param {?Element} element Element to decorate.
  * @override
  */
 goog.ui.PopupMenu.prototype.decorateInternal = function(element) {
@@ -152,8 +155,8 @@ goog.ui.PopupMenu.prototype.enterDocument = function() {
  * only be attached to an element once, since attaching the same menu for
  * multiple positions doesn't make sense.
  *
- * @param {Element} element Element whose click event should trigger the menu.
- * @param {goog.positioning.Corner=} opt_targetCorner Corner of the target that
+ * @param {?Element} element Element whose click event should trigger the menu.
+ * @param {?goog.positioning.Corner=} opt_targetCorner Corner of the target that
  *     the menu should be anchored to.
  * @param {goog.positioning.Corner=} opt_menuCorner Corner of the menu that
  *     should be anchored.
@@ -161,7 +164,7 @@ goog.ui.PopupMenu.prototype.enterDocument = function() {
  *     {@link goog.events.EventType.CONTEXTMENU} events, false if it should
  *     show on {@link goog.events.EventType.MOUSEDOWN} events. Default is
  *     MOUSEDOWN.
- * @param {goog.math.Box=} opt_margin Margin for the popup used in positioning
+ * @param {?goog.math.Box=} opt_margin Margin for the popup used in positioning
  *     algorithms.
  */
 goog.ui.PopupMenu.prototype.attach = function(
@@ -178,6 +181,54 @@ goog.ui.PopupMenu.prototype.attach = function(
   if (this.isInDocument()) {
     this.attachEvent_(target);
   }
+
+  // Add a listener for keyboard actions on the menu.
+  var handler = goog.partial(this.onMenuKeyboardAction_, element);
+  if (this.getElement()) {
+    this.getHandler().listen(this.getElement(), goog.events.EventType.KEYDOWN,
+        handler);
+  }
+};
+
+
+/**
+ * Handles keyboard actions on the PopupMenu, according to
+ * http://www.w3.org/WAI/PF/aria-practices/#menubutton.
+ *
+ * <p>If the ESC key is pressed, the menu is hidden (which is handled by
+ * this.onAction_), and the focus is returned to the element whose click event
+ * triggered opening of the menu.
+ *
+ * <p>If the SPACE or ENTER keys are pressed, the highlighted menu item's
+ * listeners are fired.
+ *
+ * @param {Element} element Element whose click event triggered the menu.
+ * @param {!goog.events.BrowserEvent} e The key down event.
+ * @private
+ */
+goog.ui.PopupMenu.prototype.onMenuKeyboardAction_ = function(element, e) {
+  if (e.keyCode == goog.events.KeyCodes.ESC) {
+    element.focus();
+    return;
+  }
+  var highlightedItem = this.getChildAt(this.getHighlightedIndex());
+  var targetElement = highlightedItem.getElement();
+  // Create an event to pass to the menu item's listener.
+  var event = new goog.events.BrowserEvent(e.getBrowserEvent(), targetElement);
+  event.target = targetElement;
+  // If an item is highlighted and the user presses the SPACE/ENTER key, the
+  // event target is the menu rather than the menu item, so we manually fire
+  // the listener of the correct menu item.
+  if (e.keyCode == goog.events.KeyCodes.SPACE ||
+      e.keyCode == goog.events.KeyCodes.ENTER) {
+    goog.events.fireListeners(targetElement,
+        goog.events.EventType.KEYDOWN, false, event);
+  }
+  // After activating a menu item the PopupMenu should be hidden (already
+  // implemented in this.onAction_ for ENTER/MOUSEDOWN).
+  if (e.keyCode == goog.events.KeyCodes.SPACE) {
+    this.hide();
+  }
 };
 
 
@@ -189,19 +240,19 @@ goog.ui.PopupMenu.prototype.attach = function(
  *
  * Subclass may add more properties to the returned object, as needed.
  *
- * @param {Element} element Element whose click event should trigger the menu.
- * @param {goog.positioning.Corner=} opt_targetCorner Corner of the target that
+ * @param {?Element} element Element whose click event should trigger the menu.
+ * @param {?goog.positioning.Corner=} opt_targetCorner Corner of the target that
  *     the menu should be anchored to.
- * @param {goog.positioning.Corner=} opt_menuCorner Corner of the menu that
+ * @param {?goog.positioning.Corner=} opt_menuCorner Corner of the menu that
  *     should be anchored.
  * @param {boolean=} opt_contextMenu Whether the menu should show on
  *     {@link goog.events.EventType.CONTEXTMENU} events, false if it should
  *     show on {@link goog.events.EventType.MOUSEDOWN} events. Default is
  *     MOUSEDOWN.
- * @param {goog.math.Box=} opt_margin Margin for the popup used in positioning
+ * @param {?goog.math.Box=} opt_margin Margin for the popup used in positioning
  *     algorithms.
  *
- * @return {Object} An object that describes how the popup menu should be
+ * @return {?Object} An object that describes how the popup menu should be
  *     attached to the anchoring element.
  *
  * @protected
@@ -232,8 +283,8 @@ goog.ui.PopupMenu.prototype.createAttachTarget = function(
  * element or {@code null}. The object is created and the association is formed
  * when {@link #attach} is invoked.
  *
- * @param {Element} element DOM element.
- * @return {Object} The object created when {@link attach} is invoked on
+ * @param {?Element} element DOM element.
+ * @return {?Object} The object created when {@link attach} is invoked on
  *     {@code element}. Returns {@code null} if the element does not trigger
  *     the menu (i.e. {@link attach} has never been invoked on
  *     {@code element}).
@@ -241,13 +292,13 @@ goog.ui.PopupMenu.prototype.createAttachTarget = function(
  */
 goog.ui.PopupMenu.prototype.getAttachTarget = function(element) {
   return element ?
-      /** @type {Object} */(this.targets_.get(goog.getUid(element))) :
+      /** @type {?Object} */(this.targets_.get(goog.getUid(element))) :
       null;
 };
 
 
 /**
- * @param {Element} element Any DOM element.
+ * @param {?Element} element Any DOM element.
  * @return {boolean} Whether clicking on the given element will trigger the
  *     menu.
  *
@@ -259,7 +310,7 @@ goog.ui.PopupMenu.prototype.isAttachTarget = function(element) {
 
 
 /**
- * @return {Element} The current element where the popup is anchored, if it's
+ * @return {?Element} The current element where the popup is anchored, if it's
  *     visible.
  */
 goog.ui.PopupMenu.prototype.getAttachedElement = function() {
@@ -268,13 +319,19 @@ goog.ui.PopupMenu.prototype.getAttachedElement = function() {
 
 
 /**
- * Attaches an event listener to a target
- * @param {Object} target The target to attach an event to.
+ * Attaches two event listeners to a target. One with corresponding event type,
+ * and one with the KEYDOWN event type for accessibility purposes.
+ * @param {?Object} target The target to attach an event to.
  * @private
  */
 goog.ui.PopupMenu.prototype.attachEvent_ = function(target) {
   this.getHandler().listen(
       target.element_, target.eventType_, this.onTargetClick_);
+  if (target.eventType_ != goog.events.EventType.CONTEXTMENU) {
+    this.getHandler().listen(
+        target.element_, goog.events.EventType.KEYDOWN,
+        this.onTargetKeyboardAction_);
+  }
 };
 
 
@@ -285,7 +342,7 @@ goog.ui.PopupMenu.prototype.detachAll = function() {
   if (this.isInDocument()) {
     var keys = this.targets_.getKeys();
     for (var i = 0; i < keys.length; i++) {
-      this.detachEvent_(/** @type {Object} */ (this.targets_.get(keys[i])));
+      this.detachEvent_(/** @type {!Object} */ (this.targets_.get(keys[i])));
     }
   }
 
@@ -295,7 +352,7 @@ goog.ui.PopupMenu.prototype.detachAll = function() {
 
 /**
  * Detaches a menu from a given element.
- * @param {Element} element Element whose click event should trigger the menu.
+ * @param {?Element} element Element whose click event should trigger the menu.
  */
 goog.ui.PopupMenu.prototype.detach = function(element) {
   if (!this.isAttachTarget(element)) {
@@ -304,7 +361,7 @@ goog.ui.PopupMenu.prototype.detach = function(element) {
 
   var key = goog.getUid(element);
   if (this.isInDocument()) {
-    this.detachEvent_(/** @type {Object} */ (this.targets_.get(key)));
+    this.detachEvent_(/** @type {!Object} */ (this.targets_.get(key)));
   }
 
   this.targets_.remove(key);
@@ -313,7 +370,7 @@ goog.ui.PopupMenu.prototype.detach = function(element) {
 
 /**
  * Detaches an event listener to a target
- * @param {Object} target The target to detach events from.
+ * @param {!Object} target The target to detach events from.
  * @private
  */
 goog.ui.PopupMenu.prototype.detachEvent_ = function(target) {
@@ -343,12 +400,13 @@ goog.ui.PopupMenu.prototype.getToggleMode = function() {
 
 /**
  * Show the menu using given positioning object.
- * @param {goog.positioning.AbstractPosition} position The positioning instance.
+ * @param {?goog.positioning.AbstractPosition} position The positioning
+ *     instance.
  * @param {goog.positioning.Corner=} opt_menuCorner The corner of the menu to be
  *     positioned.
- * @param {goog.math.Box=} opt_margin A margin specified in pixels.
- * @param {Element=} opt_anchor The element which acts as visual anchor for this
- *     menu.
+ * @param {?goog.math.Box=} opt_margin A margin specified in pixels.
+ * @param {?Element=} opt_anchor The element which acts as visual anchor for
+ *     this menu.
  */
 goog.ui.PopupMenu.prototype.showWithPosition = function(position,
     opt_menuCorner, opt_margin, opt_anchor) {
@@ -398,7 +456,7 @@ goog.ui.PopupMenu.prototype.showWithPosition = function(position,
 
 /**
  * Show the menu at a given attached target.
- * @param {Object} target Popup target.
+ * @param {!Object} target Popup target.
  * @param {number} x The client-X associated with the show event.
  * @param {number} y The client-Y associated with the show event.
  * @protected
@@ -435,7 +493,7 @@ goog.ui.PopupMenu.prototype.showAt = function(x, y, opt_menuCorner) {
 
 /**
  * Shows the menu immediately attached to the given element
- * @param {Element} element The element to show at.
+ * @param {?Element} element The element to show at.
  * @param {goog.positioning.Corner} targetCorner The corner of the target to
  *     anchor to.
  * @param {goog.positioning.Corner=} opt_menuCorner Corner of the menu that
@@ -491,7 +549,7 @@ goog.ui.PopupMenu.prototype.wasRecentlyHidden = function() {
 
 /**
  * Dismiss the popup menu when an action fires.
- * @param {goog.events.Event=} opt_e The optional event.
+ * @param {?goog.events.Event=} opt_e The optional event.
  * @private
  */
 goog.ui.PopupMenu.prototype.onAction_ = function(opt_e) {
@@ -500,18 +558,45 @@ goog.ui.PopupMenu.prototype.onAction_ = function(opt_e) {
 
 
 /**
- * Handles a browser event on one of the popup targets
- * @param {goog.events.BrowserEvent} e The browser event.
+ * Handles a browser click event on one of the popup targets.
+ * @param {?goog.events.BrowserEvent} e The browser event.
  * @private
  */
 goog.ui.PopupMenu.prototype.onTargetClick_ = function(e) {
+  this.onTargetActivation_(e);
+};
+
+
+/**
+ * Handles a KEYDOWN browser event on one of the popup targets.
+ * @param {!goog.events.BrowserEvent} e The browser event.
+ * @private
+ */
+goog.ui.PopupMenu.prototype.onTargetKeyboardAction_ = function(e) {
+  if (e.keyCode == goog.events.KeyCodes.SPACE ||
+      e.keyCode == goog.events.KeyCodes.ENTER ||
+      e.keyCode == goog.events.KeyCodes.DOWN) {
+    this.onTargetActivation_(e);
+  }
+  // If the popupmenu is opened using the DOWN key, the focus should be on the
+  // first menu item.
+  if (e.keyCode == goog.events.KeyCodes.DOWN) {
+    this.highlightFirst();
+  }
+};
+
+
+/**
+ * Handles a browser event on one of the popup targets.
+ * @param {?goog.events.BrowserEvent} e The browser event.
+ * @private
+ */
+goog.ui.PopupMenu.prototype.onTargetActivation_ = function(e) {
   var keys = this.targets_.getKeys();
   for (var i = 0; i < keys.length; i++) {
-    var target = /** @type {Object} */(this.targets_.get(keys[i]));
+    var target = /** @type {!Object} */(this.targets_.get(keys[i]));
     if (target.element_ == e.currentTarget) {
-      this.showMenu(target,
-                    /** @type {number} */ (e.clientX),
-                    /** @type {number} */ (e.clientY));
+      this.showMenu(target, (e.clientX), (e.clientY));
       e.preventDefault();
       e.stopPropagation();
       return;
@@ -522,12 +607,12 @@ goog.ui.PopupMenu.prototype.onTargetClick_ = function(e) {
 
 /**
  * Handles click events that propagate to the document.
- * @param {goog.events.BrowserEvent} e The browser event.
+ * @param {!goog.events.BrowserEvent} e The browser event.
  * @protected
  */
 goog.ui.PopupMenu.prototype.onDocClick = function(e) {
   if (this.isVisible() &&
-      !this.containsElement(/** @type {Element} */ (e.target))) {
+      !this.containsElement(/** @type {!Element} */ (e.target))) {
     this.hide();
   }
 };
@@ -535,7 +620,7 @@ goog.ui.PopupMenu.prototype.onDocClick = function(e) {
 
 /**
  * Handles the key event target loosing focus.
- * @param {goog.events.BrowserEvent} e The browser event.
+ * @param {?goog.events.BrowserEvent} e The browser event.
  * @protected
  * @override
  */
