@@ -112,6 +112,15 @@ function testRelativeUrisHaveNoPath() {
 }
 
 
+function testReservedCharacters() {
+  var o = '%6F';
+  var uri = 'http://www.g' + o + 'ogle.com%40/xxx%2feee/ccc';
+  assertEquals('Should not decode reserved characters in path',
+      '/xxx%2feee/ccc', goog.uri.utils.getPath(uri));
+  assertEquals('Should not decode reserved characters in domain',
+      'www.google.com%40', goog.uri.utils.getDomain(uri));
+}
+
 function testSetFragmentEncoded() {
   var expected = 'http://www.google.com/path#bar';
   assertEquals(expected,
@@ -564,3 +573,32 @@ function testMakeUnique() {
       goog.uri.utils.makeUnique('http://www.google.com?zx=9&a=1&b=2#blob'));
 }
 
+
+function testParseQuery() {
+  var result = [];
+  goog.uri.utils.parseQueryData(
+      'foo=bar&no&empty=&tricky%3D%26=%3D%26&=nothing&=&',
+      function(name, value) { result.push(name, value); });
+  assertArrayEquals(
+      ['foo', 'bar',
+       'no', '',
+       'empty', '',
+       'tricky%3D%26', '=&',
+       '', 'nothing',
+       '', '',
+       '', ''],
+      result);
+
+  // Go thought buildQueryData and parseQueryData and see if we get the same
+  // result.
+  var result2 = [];
+  goog.uri.utils.parseQueryData(
+      goog.uri.utils.buildQueryData(result),
+      function(name, value) { result2.push(name, value); });
+  assertArrayEquals(result, result2);
+
+  goog.uri.utils.parseQueryData('',
+      goog.partial(fail, 'Empty string should not run callback'));
+}
+
+
